@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+    Copyright (C) 1998, 1999, 2000, 2002, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,15 +13,12 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef PED_DEBUG_H_INCLUDED
 #define PED_DEBUG_H_INCLUDED
 
-#include <config.h>
-
 #include <stdarg.h>
 
 #ifdef DEBUG
@@ -60,12 +57,12 @@ extern int ped_assert ( int cond, const char* cond_text,
 static void PED_DEBUG (int level, ...)
 {
         va_list         va_args;
-        
+
         va_start (va_args, level);
         ped_debug ( level, "unknown file", 0, "unknown function", va_args );
         va_end (va_args);
 }
-        
+
 #define PED_ASSERT(cond, action)				\
 	do {							\
 	if (!ped_assert ( cond,                     		\
