@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2002, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002, 2007, 2008 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /**
@@ -27,20 +26,6 @@
 #ifndef PED_DISK_H_INCLUDED
 #define PED_DISK_H_INCLUDED
 
-typedef enum _PedPartitionType          PedPartitionType;
-typedef enum _PedPartitionFlag          PedPartitionFlag;
-typedef enum _PedDiskTypeFeature        PedDiskTypeFeature;
-typedef struct _PedDisk                 PedDisk;
-typedef struct _PedPartition            PedPartition;
-typedef const struct _PedDiskOps        PedDiskOps;
-typedef struct _PedDiskType             PedDiskType;
-typedef const struct _PedDiskArchOps    PedDiskArchOps;
-
-#include <parted/device.h>
-#include <parted/filesys.h>
-#include <parted/natmath.h>
-#include <parted/geom.h>
-
 /**
  * Partition types
  */
@@ -67,10 +52,11 @@ enum _PedPartitionFlag {
         PED_PARTITION_HPSERVICE=8,
         PED_PARTITION_PALO=9,
         PED_PARTITION_PREP=10,
-        PED_PARTITION_MSFT_RESERVED=11
+        PED_PARTITION_MSFT_RESERVED=11,
+        PED_PARTITION_BIOS_GRUB=12
 };
 #define PED_PARTITION_FIRST_FLAG        PED_PARTITION_BOOT
-#define PED_PARTITION_LAST_FLAG         PED_PARTITION_MSFT_RESERVED
+#define PED_PARTITION_LAST_FLAG         PED_PARTITION_BIOS_GRUB
 
 enum _PedDiskTypeFeature {
         PED_DISK_TYPE_EXTENDED=1,       /**< supports extended partitions */
@@ -79,11 +65,32 @@ enum _PedDiskTypeFeature {
 #define PED_DISK_TYPE_FIRST_FEATURE    PED_DISK_TYPE_EXTENDED
 #define PED_DISK_TYPE_LAST_FEATURE     PED_DISK_TYPE_PARTITION_NAME
 
+struct _PedDisk;
+struct _PedPartition;
+struct _PedDiskOps;
+struct _PedDiskType;
+struct _PedDiskArchOps;
+
+typedef enum _PedPartitionType          PedPartitionType;
+typedef enum _PedPartitionFlag          PedPartitionFlag;
+typedef enum _PedDiskTypeFeature        PedDiskTypeFeature;
+typedef struct _PedDisk                 PedDisk;
+typedef struct _PedPartition            PedPartition;
+typedef const struct _PedDiskOps        PedDiskOps;
+typedef struct _PedDiskType             PedDiskType;
+typedef const struct _PedDiskArchOps    PedDiskArchOps;
+
+#include <parted/device.h>
+#include <parted/filesys.h>
+#include <parted/natmath.h>
+#include <parted/geom.h>
+#include <stdbool.h>
+
 /** @} */
 
 /**
  * \addtogroup PedPartition
- * 
+ *
  * @{
  */
 
@@ -102,22 +109,22 @@ struct _PedPartition {
 
         /**< the partition number:  In Linux, this is the
              same as the minor number. No assumption
-             should be made about "num" and "type" 
+             should be made about "num" and "type"
              - different disk labels have different rules. */
 
         int                     num;
         PedPartitionType        type;	/**< the type of partition: a bit field of
 					  	PED_PARTITION_LOGICAL, PED_PARTITION_EXTENDED,
-						PED_PARTITION_METADATA 
-						and PED_PARTITION_FREESPACE.  
-						Both the first two, and the last two are 
+						PED_PARTITION_METADATA
+						and PED_PARTITION_FREESPACE.
+						Both the first two, and the last two are
 						mutually exclusive.
-							An extended partition is a primary 
+							An extended partition is a primary
 						partition that may contain logical partitions.
-						There is at most one extended partition on 
+						There is at most one extended partition on
 						a disk.
-							A logical partition is like a primary 
-						partition, except it's inside an extended 
+							A logical partition is like a primary
+						partition, except it's inside an extended
 						partition. Internally, pseudo partitions are
 						allocated to represent free space, or disk
 						label meta-data.  These have the
@@ -173,7 +180,7 @@ struct _PedDiskOps {
         int (*read) (PedDisk* disk);
         int (*write) (const PedDisk* disk);
         /** \todo add label guessing op here */
-        
+
         /* partition operations */
         PedPartition* (*partition_new) (
                 const PedDisk* disk,
@@ -204,6 +211,8 @@ struct _PedDiskOps {
         /* other */
         int (*alloc_metadata) (PedDisk* disk);
         int (*get_max_primary_partition_count) (const PedDisk* disk);
+        bool (*get_max_supported_partition_count) (const PedDisk* disk,
+                                                   int* supported);
 };
 
 struct _PedDiskType {
@@ -228,18 +237,6 @@ struct _PedDiskArchOps {
 extern void ped_disk_type_register (PedDiskType* type);
 extern void ped_disk_type_unregister (PedDiskType* type);
 
-/**
- * Deprecated: use ped_disk_type_register.
- */
-__attribute__ ((deprecated))
-extern void ped_register_disk_type (PedDiskType* type);
-
-/**
- * Deprecated: use ped_disk_type_unregister.
- */
-__attribute__ ((deprecated))
-extern void ped_unregister_disk_type (PedDiskType* type);
-
 extern PedDiskType* ped_disk_type_get_next (PedDiskType* type);
 extern PedDiskType* ped_disk_type_get (const char* name);
 extern int ped_disk_type_check_feature (const PedDiskType* disk_type,
@@ -263,12 +260,14 @@ extern void ped_disk_print (const PedDisk* disk);
 extern int ped_disk_get_primary_partition_count (const PedDisk* disk);
 extern int ped_disk_get_last_partition_num (const PedDisk* disk);
 extern int ped_disk_get_max_primary_partition_count (const PedDisk* disk);
+extern bool ped_disk_get_max_supported_partition_count(const PedDisk* disk,
+                                                       int* supported);
 
 /** @} */
 
 /**
  * \addtogroup PedPartition
- * 
+ *
  * @{
  */
 
