@@ -1,10 +1,9 @@
-/*
-    libparted - a library for manipulating disk partitions
-    Copyright (C) 1999 - 2005, 2007 Free Software Foundation, Inc.
+/* libparted - a library for manipulating disk partitions
+    Copyright (C) 1999 - 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,17 +12,16 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define PROC_DEVICES_BUFSIZ 16384
 
 #include <config.h>
+#include <arch/linux.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
-#include <parted/linux.h>
 
 #include <ctype.h>
 #include <errno.h>
@@ -32,6 +30,7 @@
 #include <stdio.h>
 #include <syscall.h>
 #include <unistd.h>
+#include <stdbool.h>
 #include <dirent.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
@@ -43,6 +42,8 @@
 #endif
 
 #include "blkpg.h"
+#include "../architecture.h"
+#include "dirname.h"
 
 #if ENABLE_NLS
 #  include <libintl.h>
@@ -65,15 +66,15 @@
 #define HDIO_GETGEO             0x0301  /* get device geometry */
 #define HDIO_GET_IDENTITY       0x030d  /* get IDE identification info */
 
-//#if defined(O_DIRECT) && (!defined(__s390__) || !defined(__s390x__))
-//#define RD_MODE (O_RDONLY | O_DIRECT)
-//#define WR_MODE (O_WRONLY | O_DIRECT)
-//#define RW_MODE (O_RDWR | O_DIRECT)
-//#else
+#if defined(O_DIRECT) && !(defined(__s390__) || defined(__s390x__))
+#define RD_MODE (O_RDONLY | O_DIRECT)
+#define WR_MODE (O_WRONLY | O_DIRECT)
+#define RW_MODE (O_RDWR | O_DIRECT)
+#else
 #define RD_MODE (O_RDONLY)
 #define WR_MODE (O_WRONLY)
 #define RW_MODE (O_RDWR)
-//#endif
+#endif
 
 struct hd_geometry {
         unsigned char heads;
@@ -254,15 +255,111 @@ struct blkdev_ioctl_param {
 #define VIODASD_MAJOR           112
 #define SX8_MAJOR1              160
 #define SX8_MAJOR2              161
+#define XVD_MAJOR               202
+#define SDMMC_MAJOR             179
 
 #define SCSI_BLK_MAJOR(M) (                                             \
                 (M) == SCSI_DISK0_MAJOR                                 \
                 || (M) == SCSI_CDROM_MAJOR                              \
                 || ((M) >= SCSI_DISK1_MAJOR && (M) <= SCSI_DISK7_MAJOR))
 
+/* Maximum number of partitions supported by linux. */
+#define MAX_NUM_PARTS		64
+
 static char* _device_get_part_path (PedDevice* dev, int num);
 static int _partition_is_mounted_by_path (const char* path);
 
+static int
+_read_fd (int fd, char **buf)
+{
+        char* p;
+        size_t size = PROC_DEVICES_BUFSIZ;
+        int s, filesize = 0;
+
+        *buf = malloc (size * sizeof (char));
+        if (*buf == 0) {
+                return -1;
+        }
+
+        do {
+                p = &(*buf) [filesize];
+                s = read (fd, p, PROC_DEVICES_BUFSIZ);
+                /* exit if there is an error or EOF is reached */
+                if (s <= 0)
+                        break;
+                filesize += s;
+                size += s;
+                *buf = realloc (*buf, size);
+        } while (1);
+
+        if (filesize == 0 && s < 0) {
+                free (*buf);
+                *buf = NULL;
+                return -1;
+        } else {
+                /* there is always some excess memory left unused */
+                *buf = realloc (*buf, filesize+1);
+                (*buf)[filesize] = '\0';
+        }
+
+        return filesize;
+}
+
+static int
+_major_type_in_devices (int major, const char* type)
+{
+        int fd;
+        char* buf = NULL;
+        char* line;
+        char* end;
+        int bd = 0;
+        char c;
+
+        fd = open ("/proc/devices", O_RDONLY);
+        if (fd < 0)
+                return 0;
+
+        if (_read_fd(fd, &buf) < 0) {
+                close(fd);
+                return 0;
+        }
+
+        line = buf;
+        end = strchr(line, '\n');
+        while (end) {
+                char *name;
+                int maj;
+
+                c = *end;
+                *end = '\0';
+
+                if (!bd) {
+                        if (!strncmp(line, "Block devices:", 14))
+                                bd = 1;
+                        goto next;
+                }
+
+                name = strrchr(line, ' ');
+                if (!name || strcmp(name+1, type))
+                        goto next;
+
+                maj = strtol(line, &name, 10);
+                if (maj == major) {
+                        free(buf);
+                        close(fd);
+                        return 1;
+                }
+
+next:
+                *end = c;
+                line = end+1;
+                end = strchr(line, '\n');
+        }
+        free(buf);
+        close(fd);
+        return 0;
+}
+
 static int
 _is_ide_major (int major)
 {
@@ -299,96 +396,89 @@ _is_sx8_major (int major)
         return (SX8_MAJOR1 <= major && major <= SX8_MAJOR2);
 }
 
-#ifdef ENABLE_DEVICE_MAPPER
 static int
-readFD (int fd, char **buf)
+_is_virtblk_major (int major)
 {
-        char* p;
-        size_t size = PROC_DEVICES_BUFSIZ;
-        int s, filesize = 0;
-
-        *buf = malloc (size * sizeof (char));
-        if (*buf == 0) {
-                return -1;
-        }
-
-        do {
-                p = &(*buf) [filesize];
-                s = read (fd, p, PROC_DEVICES_BUFSIZ);
-                /* exit if there is an error or EOF is reached */
-                if (s <= 0)
-                        break;
-                filesize += s;
-                size += s;
-                *buf = realloc (*buf, size);
-        } while (1);
-
-        if (filesize == 0 && s < 0) {
-                free (*buf);
-                *buf = NULL;
-                return -1;
-        } else {
-                /* there is always some excess memory left unused */
-                *buf = realloc (*buf, filesize+1);
-                (*buf)[filesize] = '\0';
-        }
-
-        return filesize;
+        return _major_type_in_devices (major, "virtblk");
 }
 
+#ifdef ENABLE_DEVICE_MAPPER
 static int
 _is_dm_major (int major)
 {
-        int fd;
-        char* buf = NULL;
-        char* line;
-        char* end;
-        int bd = 0;
-        char c;
+        return _major_type_in_devices (major, "device-mapper");
+}
 
-        fd = open ("/proc/devices", O_RDONLY);
-        if (fd < 0)
-                return 0;
+static int
+_dm_maptype (PedDevice *dev)
+{
+        LinuxSpecific*  arch_specific = LINUX_SPECIFIC (dev);
+        struct dm_task *dmt;
+        void *next;
+        uint64_t start, length;
+        char *target_type = NULL;
+        char *params;
+        int r = -1;
+        const char* dev_dir = getenv ("DM_DEV_DIR");
 
-        if (readFD(fd, &buf) < 0) {
-                close(fd);
-                return 0;
-        }
+        if (dev_dir && *dev_dir && !dm_set_dev_dir(dev_dir))
+                return r;
 
-        line = buf;
-        end = strchr(line, '\n');
-        while (end) {
-                char *name;
-                int maj;
+        if (!(dmt = dm_task_create(DM_DEVICE_TABLE)))
+                return r;
 
-                c = *end;
-                *end = '\0';
+        if (!dm_task_set_name(dmt, dev->path))
+                goto bad;
 
-                if (!bd) {
-                        if (!strncmp(line, "Block devices:", 14))
-                                bd = 1;
-                        goto next;
-                }
+        dm_task_no_open_count(dmt);
 
-                name = strrchr(line, ' ');
-                if (!name || strcmp(name+1, "device-mapper"))
-                        goto next;
+        if (!dm_task_run(dmt))
+                goto bad;
 
-                maj = strtol(line, &name, 10);
-                if (maj == major) {
-                        free(buf);
-                        close(fd);
-                        return 1;
-                }
+        next = dm_get_next_target(dmt, NULL, &start, &length,
+                                  &target_type, &params);
 
-next:
-                *end = c;
-                line = end+1;
-                end = strchr(line, '\n');
-        }
-        free(buf);
-        close(fd);
-        return 0;
+        arch_specific->dmtype = strdup(target_type ? target_type : "NO-TARGET");
+        if (arch_specific->dmtype == NULL)
+                goto bad;
+        r = 0;
+bad:
+        dm_task_destroy(dmt);
+        return r;
+}
+
+
+static int
+_probe_dm_devices ()
+{
+       DIR*            mapper_dir;
+       struct dirent*  dent;
+       char            buf [512];      /* readdir(3) claims d_name[256] */
+       struct stat     st;
+
+       mapper_dir = opendir ("/dev/mapper");
+       if (!mapper_dir)
+               return 0;
+
+       /* Search the /dev/mapper directory for devices w/ the same major
+        * number that was returned from _probe_lvm_major().
+        */
+       while ((dent = readdir (mapper_dir))) {
+               if (strcmp (dent->d_name, ".")  == 0 ||
+                   strcmp (dent->d_name, "..") == 0)
+                       continue;
+
+               snprintf (buf, sizeof (buf), "/dev/mapper/%s", dent->d_name);
+
+               if (stat (buf, &st) != 0)
+                       continue;
+
+               if (_is_dm_major(major(st.st_rdev)))
+                       _ped_device_probe (buf);
+       }
+       closedir (mapper_dir);
+
+       return 1;
 }
 #endif
 
@@ -455,7 +545,20 @@ _device_probe_type (PedDevice* dev)
 #ifdef ENABLE_DEVICE_MAPPER
         } else if (_is_dm_major(dev_major)) {
                 dev->type = PED_DEVICE_DM;
+                if (_dm_maptype(dev)) {
+                        ped_exception_throw (
+                                PED_EXCEPTION_BUG,
+                                PED_EXCEPTION_CANCEL,
+                                _("Unable to determine the dm type of %s."),
+                                dev->path);
+                }
 #endif
+        } else if (dev_major == XVD_MAJOR && (dev_minor % 0x10 == 0)) {
+                dev->type = PED_DEVICE_XVD;
+        } else if (dev_major == SDMMC_MAJOR && (dev_minor % 0x08 == 0)) {
+                dev->type = PED_DEVICE_SDMMC;
+        } else if (_is_virtblk_major(dev_major)) {
+                dev->type = PED_DEVICE_VIRTBLK;
         } else {
                 dev->type = PED_DEVICE_UNKNOWN;
         }
@@ -500,12 +603,12 @@ _have_devfs ()
         return have_devfs = S_ISCHR(sb.st_mode) ? 1 : 0;
 }
 
-static void 
+static void
 _device_set_sector_size (PedDevice* dev)
 {
         LinuxSpecific*  arch_specific = LINUX_SPECIFIC (dev);
         int sector_size;
-        
+
         dev->sector_size = PED_SECTOR_SIZE_DEFAULT;
         dev->phys_sector_size = PED_SECTOR_SIZE_DEFAULT;
 
@@ -515,7 +618,7 @@ _device_set_sector_size (PedDevice* dev)
                 dev->sector_size = PED_SECTOR_SIZE_DEFAULT;
                 return;
         }
-        
+
         if (ioctl (arch_specific->fd, BLKSSZGET, &sector_size)) {
                 ped_exception_throw (
                         PED_EXCEPTION_WARNING,
@@ -597,7 +700,7 @@ _device_probe_geometry (PedDevice* dev)
         PED_ASSERT (S_ISBLK (dev_stat.st_mode), return 0);
 
         _device_set_sector_size (dev);
-        
+
         dev->length = _device_get_length (dev);
         if (!dev->length)
                 return 0;
@@ -652,7 +755,7 @@ init_ide (PedDevice* dev)
         PedExceptionOption      ex_status;
         char                    hdi_buf[41];
         int                     sector_multiplier = 0;
-        
+
         if (!_device_stat (dev, &dev_stat))
                 goto error;
 
@@ -685,12 +788,12 @@ init_ide (PedDevice* dev)
                 memcpy (hdi_buf, hdi.model, 40);
                 hdi_buf[40] = '\0';
                 dev->model = strip_name (hdi_buf);
-                
+
                 if (!hdi.ata7_sectinfo.valid1 && hdi.ata7_sectinfo.valid2)
                         sector_multiplier = hdi.ata7_sectinfo.multiplier;
                 else
                         sector_multiplier = 1;
-                        
+
                 if (sector_multiplier != 1) {
                         ex_status = ped_exception_throw (
                                 PED_EXCEPTION_WARNING,
@@ -703,7 +806,7 @@ init_ide (PedDevice* dev)
                                   "Please consult the web site for up-to-date "
                                   "information."),
                                 dev->path, sector_multiplier);
-                        
+
                         switch (ex_status) {
                                 case PED_EXCEPTION_CANCEL:
                                         goto error_close_dev;
@@ -717,7 +820,7 @@ init_ide (PedDevice* dev)
                                         break;
                         }
                 }
-                
+
                 /* XXX sector_size has not been set yet! */
                 /* dev->phys_sector_size = dev->sector_size
                    * sector_multiplier;*/
@@ -745,7 +848,7 @@ read_device_sysfs_file (PedDevice *dev, const char *file)
         char buf[256];
 
         snprintf (name_buf, 127, "/sys/block/%s/device/%s",
-                  basename (dev->path), file);
+                  last_component (dev->path), file);
 
         if ((f = fopen (name_buf, "r")) == NULL)
                 return NULL;
@@ -889,8 +992,8 @@ init_scsi (PedDevice* dev)
 
         if (scsi_get_product_info (dev, &vendor, &product)) {
                 sprintf (dev->model, "%.8s %.16s", vendor, product);
-                ped_free (vendor);
-                ped_free (product);
+                free (vendor);
+                free (product);
         } else {
                 strcpy (dev->model, "Generic SCSI");
         }
@@ -911,7 +1014,7 @@ static int
 init_file (PedDevice* dev)
 {
         struct stat     dev_stat;
- 
+
         if (!_device_stat (dev, &dev_stat))
                 goto error;
         if (!ped_device_open (dev))
@@ -941,9 +1044,9 @@ init_file (PedDevice* dev)
         dev->sector_size = PED_SECTOR_SIZE_DEFAULT;
         dev->phys_sector_size = PED_SECTOR_SIZE_DEFAULT;
         dev->model = strdup ("");
-        
+
         return 1;
-        
+
 error_close_dev:
         ped_device_close (dev);
 error:
@@ -951,7 +1054,7 @@ error:
 }
 
 static int
-init_dasd (PedDevice* dev, char* model_name)
+init_dasd (PedDevice* dev, const char* model_name)
 {
         struct stat             dev_stat;
         struct hd_geometry      geo;
@@ -1007,7 +1110,7 @@ error:
 }
 
 static int
-init_generic (PedDevice* dev, char* model_name)
+init_generic (PedDevice* dev, const char* model_name)
 {
         struct stat             dev_stat;
         PedExceptionOption      ex_status;
@@ -1066,10 +1169,39 @@ error:
         return 0;
 }
 
+static int
+sdmmc_get_product_info (PedDevice* dev, char **type, char **name)
+{
+        *type = read_device_sysfs_file (dev, "type");
+        *name = read_device_sysfs_file (dev, "name");
+        if (*type && *name)
+                return 1;
+
+        return 0;
+}
+
+static int
+init_sdmmc (PedDevice* dev)
+{
+        char id[128];
+        char *type, *name;
+
+        if (sdmmc_get_product_info (dev, &type, &name)) {
+                snprintf (id, sizeof(id) - 1, "%s %s", type, name);
+                free (type);
+                free (name);
+        } else {
+                snprintf (id, sizeof(id) - 1, "%s",
+                          _("Generic SD/MMC Storage Card"));
+        }
+        return init_generic(dev, id);
+}
+
 static PedDevice*
 linux_new (const char* path)
 {
         PedDevice*      dev;
+        LinuxSpecific*  arch_specific;
 
         PED_ASSERT (path != NULL, return NULL);
 
@@ -1085,6 +1217,8 @@ linux_new (const char* path)
                 = (LinuxSpecific*) ped_malloc (sizeof (LinuxSpecific));
         if (!dev->arch_specific)
                 goto error_free_path;
+        arch_specific = LINUX_SPECIFIC (dev);
+        arch_specific->dmtype = NULL;
 
         dev->open_count = 0;
         dev->read_only = 0;
@@ -1151,18 +1285,39 @@ linux_new (const char* path)
                         goto error_free_arch_specific;
                 break;
 
-#ifdef ENABLE_DEVICE_MAPPER
         case PED_DEVICE_DM:
-                if (!init_generic (dev, _("Linux device-mapper")))
+                {
+                  char* type;
+                  if (arch_specific->dmtype == NULL
+                      || asprintf(&type, _("Linux device-mapper (%s)"),
+                                  arch_specific->dmtype) == -1)
+                        goto error_free_arch_specific;
+                  bool ok = init_generic (dev, type);
+                  free (type);
+                  if (!ok)
+                    goto error_free_arch_specific;
+                  break;
+                }
+
+        case PED_DEVICE_XVD:
+                if (!init_generic (dev, _("Xen Virtual Block Device")))
                         goto error_free_arch_specific;
                 break;
-#endif
 
         case PED_DEVICE_UNKNOWN:
                 if (!init_generic (dev, _("Unknown")))
                         goto error_free_arch_specific;
                 break;
 
+        case PED_DEVICE_SDMMC:
+                if (!init_sdmmc (dev))
+                        goto error_free_arch_specific;
+                break;
+        case PED_DEVICE_VIRTBLK:
+                if (!init_generic(dev, _("Virtio Block Device")))
+                        goto error_free_arch_specific;
+                break;
+
         default:
                 ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
                                 PED_EXCEPTION_CANCEL,
@@ -1172,11 +1327,11 @@ linux_new (const char* path)
         return dev;
 
 error_free_arch_specific:
-        ped_free (dev->arch_specific);
+        free (dev->arch_specific);
 error_free_path:
-        ped_free (dev->path);
+        free (dev->path);
 error_free_dev:
-        ped_free (dev);
+        free (dev);
 error:
         return NULL;
 }
@@ -1184,10 +1339,11 @@ error:
 static void
 linux_destroy (PedDevice* dev)
 {
-        ped_free (dev->arch_specific);
-        ped_free (dev->path);
-        ped_free (dev->model);
-        ped_free (dev);
+        free (((LinuxSpecific*)dev->arch_specific)->dmtype);
+        free (dev->arch_specific);
+        free (dev->path);
+        free (dev->model);
+        free (dev);
 }
 
 static int
@@ -1206,7 +1362,7 @@ linux_is_busy (PedDevice* dev)
                 if (!part_name)
                         return 1;
                 status = _partition_is_mounted_by_path (part_name);
-                ped_free (part_name);
+                free (part_name);
 
                 if (status)
                         return 1;
@@ -1248,7 +1404,7 @@ _flush_cache (PedDevice* dev)
                                 close (fd);
                         }
                 }
-                ped_free (name);
+                free (name);
         }
 }
 
@@ -1348,11 +1504,11 @@ static int
 _device_seek (const PedDevice* dev, PedSector sector)
 {
         LinuxSpecific*  arch_specific;
-        
+
         PED_ASSERT (dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0, return 0);
         PED_ASSERT (dev != NULL, return 0);
         PED_ASSERT (!dev->external_mode, return 0);
-        
+
         arch_specific = LINUX_SPECIFIC (dev);
 
 #if SIZEOF_OFF_T < 8
@@ -1375,14 +1531,14 @@ _read_lastoddsector (const PedDevice* dev, void* buffer)
 
         PED_ASSERT(dev != NULL, return 0);
         PED_ASSERT(buffer != NULL, return 0);
-        
+
         arch_specific = LINUX_SPECIFIC (dev);
 
 retry:
         ioctl_param.block = 0; /* read the last sector */
         ioctl_param.content_length = dev->sector_size;
         ioctl_param.block_contents = buffer;
-        
+
         if (ioctl(arch_specific->fd, BLKGETLASTSECT, &ioctl_param) == -1) {
                 PedExceptionOption      opt;
                 opt = ped_exception_throw (
@@ -1405,10 +1561,10 @@ linux_read (const PedDevice* dev, void* buffer, PedSector start,
             PedSector count)
 {
         LinuxSpecific*          arch_specific = LINUX_SPECIFIC (dev);
-        int                     status;
         PedExceptionOption      ex_status;
-        void*                   diobuf;
+        void*                   diobuf = NULL;
 
+        PED_ASSERT (dev != NULL, return 0);
         PED_ASSERT (dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0, return 0);
 
         if (_get_linux_version() < KERNEL_VERSION (2,6,0)) {
@@ -1453,10 +1609,11 @@ linux_read (const PedDevice* dev, void* buffer, PedSector start,
                 return 0;
 
         while (1) {
-                status = read (arch_specific->fd, diobuf, read_length);
+                ssize_t status = read (arch_specific->fd, diobuf, read_length);
                 if (status > 0)
                         memcpy(buffer, diobuf, status);
-                if (status == count * dev->sector_size) break;
+                if (status == (ssize_t) read_length)
+                        break;
                 if (status > 0) {
                         read_length -= status;
                         buffer = (char *) buffer + status;
@@ -1488,7 +1645,8 @@ linux_read (const PedDevice* dev, void* buffer, PedSector start,
                                 break;
                 }
         }
-        free(diobuf);
+
+        free (diobuf);
 
         return 1;
 }
@@ -1501,14 +1659,14 @@ _write_lastoddsector (PedDevice* dev, const void* buffer)
 
         PED_ASSERT(dev != NULL, return 0);
         PED_ASSERT(buffer != NULL, return 0);
-        
+
         arch_specific = LINUX_SPECIFIC (dev);
 
 retry:
         ioctl_param.block = 0; /* write the last sector */
         ioctl_param.content_length = dev->sector_size;
         ioctl_param.block_contents = (void*) buffer;
-        
+
         if (ioctl(arch_specific->fd, BLKSETLASTSECT, &ioctl_param) == -1) {
                 PedExceptionOption      opt;
                 opt = ped_exception_throw (
@@ -1531,9 +1689,7 @@ linux_write (PedDevice* dev, const void* buffer, PedSector start,
              PedSector count)
 {
         LinuxSpecific*          arch_specific = LINUX_SPECIFIC (dev);
-        int                     status;
         PedExceptionOption      ex_status;
-        size_t                  write_length = count * dev->sector_size;
         void*                   diobuf;
         void*                   diobuf_start;
 
@@ -1591,15 +1747,15 @@ linux_write (PedDevice* dev, const void* buffer, PedSector start,
         printf ("ped_device_write (\"%s\", %p, %d, %d)\n",
                 dev->path, buffer, (int) start, (int) count);
 #else
+        size_t write_length = count * dev->sector_size;
         dev->dirty = 1;
-        if (posix_memalign(&diobuf, PED_SECTOR_SIZE_DEFAULT,
-                           count * PED_SECTOR_SIZE_DEFAULT) != 0)
+        if (posix_memalign(&diobuf, dev->sector_size, write_length) != 0)
                 return 0;
-        memcpy(diobuf, buffer, count * PED_SECTOR_SIZE_DEFAULT);
+        memcpy(diobuf, buffer, write_length);
         diobuf_start = diobuf;
         while (1) {
-                status = write (arch_specific->fd, diobuf, write_length);
-                if (status == count * dev->sector_size) break;
+                ssize_t status = write (arch_specific->fd, diobuf, write_length);
+                if (status == write_length) break;
                 if (status > 0) {
                         write_length -= status;
                         diobuf = (char *) diobuf + status;
@@ -1646,7 +1802,7 @@ linux_check (PedDevice* dev, void* buffer, PedSector start, PedSector count)
         void*           diobuf;
 
         PED_ASSERT(dev != NULL, return 0);
-        
+
         if (!_device_seek (dev, start))
                 return 0;
 
@@ -1699,7 +1855,7 @@ _do_fsync (PedDevice* dev)
                                 PED_ASSERT (0, (void) 0);
                                 break;
                 }
-        } 
+        }
         return 1;
 }
 
@@ -1821,6 +1977,7 @@ _skip_entry (const char *name)
 		{ ".",		sizeof (".") - 1	},
 		{ "..",		sizeof ("..") - 1	},
 		{ "dm-",	sizeof ("dm-") - 1	},
+		{ "md",		sizeof ("md") - 1	},
 		{ "loop",	sizeof ("loop") - 1	},
 		{ "ram",	sizeof ("ram") - 1	},
 		{ 0, 0 },
@@ -1860,7 +2017,7 @@ _probe_sys_block ()
 		}
 		_ped_device_probe (dev_name);
 	}
-	
+
 	closedir (blockdir);
 	return 1;
 }
@@ -1896,11 +2053,19 @@ linux_probe_all ()
          */
         _probe_standard_devices ();
 
- 	/* /sys/block is more reliable and consistent; fall back to using
- 	 * /proc/partitions if the former is unavailable, however.
- 	 */
- 	if (!_probe_sys_block ())
- 		_probe_proc_partitions ();
+#ifdef ENABLE_DEVICE_MAPPER
+        /* device-mapper devices aren't listed in /proc/partitions; or, if
+         * they are, they're listed as dm-X.  So, instead of relying on that,
+         * we do our own checks.
+         */
+        _probe_dm_devices ();
+#endif
+
+        /* /sys/block is more reliable and consistent; fall back to using
+         * /proc/partitions if the former is unavailable, however.
+         */
+        if (!_probe_sys_block ())
+                _probe_proc_partitions ();
 }
 
 static char*
@@ -1924,9 +2089,7 @@ _device_get_part_path (PedDevice* dev, int num)
         } else if (dev->type == PED_DEVICE_DAC960
                         || dev->type == PED_DEVICE_CPQARRAY
                         || dev->type == PED_DEVICE_ATARAID
-#ifdef ENABLE_DEVICE_MAPPER
                         || dev->type == PED_DEVICE_DM
-#endif
                         || isdigit (dev->path[path_len - 1]))
                 snprintf (result, result_len, "%sp%d", dev->path, num);
         else
@@ -2041,7 +2204,7 @@ _blkpg_part_command (PedDevice* dev, struct blkpg_partition* part, int op)
 }
 
 static int
-_blkpg_add_partition (PedDisk* disk, PedPartition* part)
+_blkpg_add_partition (PedDisk* disk, const PedPartition *part)
 {
         struct blkpg_partition  linux_part;
         const char*             vol_name;
@@ -2073,7 +2236,7 @@ _blkpg_add_partition (PedDisk* disk, PedPartition* part)
         if (vol_name)
                 strncpy (linux_part.volname, vol_name, BLKPG_VOLNAMELTH);
 
-        ped_free (dev_name);
+        free (dev_name);
 
         if (!_blkpg_part_command (disk->dev, &linux_part,
                                   BLKPG_ADD_PARTITION)) {
@@ -2105,24 +2268,78 @@ _blkpg_remove_partition (PedDisk* disk, int n)
                                     BLKPG_DEL_PARTITION);
 }
 
+/*
+ * The number of partitions that a device can have depends on the kernel.
+ * If we don't find this value in /sys/block/DEV/range, we will use our own
+ * value.
+ */
+static unsigned int
+_device_get_partition_range(PedDevice* dev)
+{
+        int         range, r;
+        char        path[128];
+        FILE*       fp;
+        bool        ok;
+
+        r = snprintf(path, sizeof(path), "/sys/block/%s/range",
+                     last_component(dev->path));
+        if(r < 0 || r >= sizeof(path))
+                return MAX_NUM_PARTS;
+
+        fp = fopen(path, "r");
+        if(!fp)
+                return MAX_NUM_PARTS;
+
+        ok = fscanf(fp, "%d", &range) == 1;
+        fclose(fp);
+
+        /* (range <= 0) is none sense.*/
+        return ok && range > 0 ? range : MAX_NUM_PARTS;
+}
+
+/*
+ * Sync the partition table in two step process:
+ * 1. Remove all of the partitions from the kernel's tables, but do not attempt
+ *    removal of any partition for which the corresponding ioctl call fails.
+ * 2. Add all the partitions that we hold in disk.
+ *
+ * To achieve this two step process we must calculate the minimum number of
+ * maximum possible partitions between what linux supports and what the label
+ * type supports. EX:
+ *
+ * number=MIN(max_parts_supported_in_linux,max_parts_supported_in_msdos_tables)
+ */
 static int
 _disk_sync_part_table (PedDisk* disk)
 {
-        int     i;
-        int     last = PED_MAX (ped_disk_get_last_partition_num (disk), 16);
-        int*    rets = ped_malloc(sizeof(int) * last);
-        int*    errnums = ped_malloc(sizeof(int) * last);
-        int     ret = 1;
+        PED_ASSERT(disk != NULL, return 0);
+        PED_ASSERT(disk->dev != NULL, return 0);
+        int lpn;
 
-        for (i = 1; i <= last; i++) {
+        /* lpn = largest partition number. */
+        if(ped_disk_get_max_supported_partition_count(disk, &lpn))
+                lpn = PED_MIN(lpn, _device_get_partition_range(disk->dev));
+        else
+                lpn = _device_get_partition_range(disk->dev);
+
+        /* Its not possible to support largest_partnum < 0.
+         * largest_partnum == 0 would mean does not support partitions.
+         * */
+        if(lpn < 0)
+                return 0;
+
+        int *rets = ped_malloc(sizeof(int) * lpn);
+        int *errnums = ped_malloc(sizeof(int) * lpn);
+        int ret = 1;
+        int i;
+
+        for (i = 1; i <= lpn; i++) {
                 rets[i - 1] = _blkpg_remove_partition (disk, i);
                 errnums[i - 1] = errno;
         }
 
-        for (i = 1; i <= last; i++) {
-                PedPartition*           part;
-
-                part = ped_disk_get_partition (disk, i);
+        for (i = 1; i <= lpn; i++) {
+                const PedPartition *part = ped_disk_get_partition (disk, i);
                 if (part) {
                         /* busy... so we won't (can't!) disturb ;)  Prolly
                          * doesn't matter anyway, because users shouldn't be
@@ -2137,6 +2354,8 @@ _disk_sync_part_table (PedDisk* disk)
                 }
         }
 
+        free (rets);
+        free (errnums);
         return ret;
 }
 
@@ -2162,7 +2381,7 @@ _dm_remove_map_name(char *name)
         return 0;
 }
 
-static int 
+static int
 _dm_is_part (struct dm_info *this, char *name)
 {
         struct dm_task* task = NULL;
@@ -2174,7 +2393,7 @@ _dm_is_part (struct dm_info *this, char *name)
         task = dm_task_create(DM_DEVICE_DEPS);
         if (!task)
                 return 0;
-        
+
         dm_task_set_name(task, name);
         rc = dm_task_run(task);
         if (rc < 0) {
@@ -2241,7 +2460,7 @@ _dm_remove_parts (PedDevice* dev)
 
         rc = 0;
         do {
-                names = (void *)names + next;
+                names = (void *) ((char *) names + next);
 
                 if (_dm_is_part(info, names->name))
                         rc += _dm_remove_map_name(names->name);
@@ -2320,8 +2539,7 @@ err:
         dm_task_update_nodes();
         if (task)
                 dm_task_destroy (task);
-        if (params)
-                free (params);
+        free (params);
         free (vol_name);
         return 0;
 }
@@ -2329,8 +2547,12 @@ err:
 static int
 _dm_reread_part_table (PedDisk* disk)
 {
+        int largest_partnum = ped_disk_get_last_partition_num (disk);
+        if (largest_partnum <= 0)
+          return 1;
+
         int     rc = 1;
-        int     last = PED_MAX (ped_disk_get_last_partition_num (disk), 16);
+        int     last = PED_MIN (largest_partnum, 16);
         int     i;
 
         sync();
