@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999 - 2001, 2005 Free Software Foundation, Inc.
+    Copyright (C) 1999 - 2001, 2005, 2007-2008 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /** \file device.c */
@@ -23,11 +22,11 @@
  * \addtogroup PedDevice
  *
  * \brief Device access.
- * 
+ *
  * When ped_device_probe_all() is called, libparted attempts to detect all
  * devices.  It constructs a list which can be accessed with
  * ped_device_get_next().
- * 
+ *
  * If you want to use a device that isn't on the list, use
  * ped_device_get().  Also, there may be OS-specific constructors, for creating
  * devices from file descriptors, stores, etc.  For example,
@@ -36,19 +35,17 @@
  * @{
  */
 
-#define _GNU_SOURCE 1
-
-#include "config.h"
+#include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
 
 #include <limits.h>
-#include <string.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 
+#include "architecture.h"
+
 static PedDevice*	devices; /* legal advice says: initialized to NULL,
 				    under section 6.7.8 part 10
 				    of ISO/EIC 9899:1999 */
@@ -185,13 +182,13 @@ ped_device_get (const char* path)
 
 	for (walk = devices; walk != NULL; walk = walk->next) {
 		if (!strcmp (walk->path, normal_path)) {
-			ped_free (normal_path);
+			free (normal_path);
 			return walk;
 		}
 	}
 
 	walk = ped_architecture->dev_ops->_new (normal_path);
-	ped_free (normal_path);
+	free (normal_path);
 	if (!walk)
 		return NULL;
 	_device_register (walk);
@@ -337,25 +334,25 @@ ped_device_end_external_access (PedDevice* dev)
 /**
  * \internal Read count sectors from dev into buffer, beginning with sector
  * start.
- * 
+ *
  * \return zero on failure.
  */
 int
 ped_device_read (const PedDevice* dev, void* buffer, PedSector start,
-		 PedSector count)
+                 PedSector count)
 {
-	PED_ASSERT (dev != NULL, return 0);
-	PED_ASSERT (buffer != NULL, return 0);
-	PED_ASSERT (!dev->external_mode, return 0);
-	PED_ASSERT (dev->open_count > 0, return 0);
+        PED_ASSERT (dev != NULL, return 0);
+        PED_ASSERT (buffer != NULL, return 0);
+        PED_ASSERT (!dev->external_mode, return 0);
+        PED_ASSERT (dev->open_count > 0, return 0);
 
-	return (ped_architecture->dev_ops->read) (dev, buffer, start, count);
+        return (ped_architecture->dev_ops->read) (dev, buffer, start, count);
 }
 
 /**
  * \internal Write count sectors from buffer to dev, starting at sector
  * start.
- * 
+ *
  * \return zero on failure.
  *
  * \sa PedDevice::sector_size
@@ -433,13 +430,17 @@ ped_device_get_constraint (PedDevice* dev)
         int multiplier = dev->phys_sector_size / dev->sector_size;
 
         PedAlignment* start_align = ped_alignment_new (multiplier, multiplier);
-        
+
+        PedGeometry *s, *e;
         PedConstraint* c = ped_constraint_new (
                                 start_align, ped_alignment_any,
-                                ped_geometry_new (dev, 0, dev->length),
-                                ped_geometry_new (dev, 0, dev->length),
+                                s = ped_geometry_new (dev, 0, dev->length),
+                                e = ped_geometry_new (dev, 0, dev->length),
                                 1, dev->length);
 
+        free (s);
+        free (e);
+        free (start_align);
         return c;
 }
 
