@@ -1,6 +1,6 @@
  /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2007
+    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2007, 2008
                   Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
  * \addtogroup PedDisk
  *
  * \brief Disk label access.
- * 
+ *
  * Most programs will need to use ped_disk_new() or ped_disk_new_fresh() to get
  * anything done.  A PedDisk is always associated with a device and has a
  * partition table.  There are different types of partition tables (or disk
@@ -36,6 +36,10 @@
 
 #include <parted/parted.h>
 #include <parted/debug.h>
+#include <stdbool.h>
+
+#include "architecture.h"
+#include "intprops.h"
 
 #if ENABLE_NLS
 #  include <libintl.h>
@@ -67,10 +71,9 @@ ped_disk_type_register (PedDiskType* disk_type)
 	PED_ASSERT (disk_type != NULL, return);
 	PED_ASSERT (disk_type->ops != NULL, return);
 	PED_ASSERT (disk_type->name != NULL, return);
-	
-	/* pretend that "next" isn't part of the struct :-) */
-	((struct _PedDiskType*) disk_type)->next = disk_types;
-	disk_types = (struct _PedDiskType*) disk_type;
+
+        disk_type->next = disk_types;
+        disk_types =  disk_type;
 }
 
 void
@@ -92,24 +95,6 @@ ped_disk_type_unregister (PedDiskType* disk_type)
 		disk_types = disk_type->next;
 }
 
-/**
- * Deprecated: use ped_disk_type_regiser.
- */
-void
-ped_register_disk_type (PedDiskType* disk_type)
-{
-        ped_disk_type_register (disk_type);
-}
-
-/**
- * Deprecated: use ped_disk_type_unregiser.
- */
-void
-ped_unregister_disk_type (PedDiskType* disk_type)
-{
-        ped_disk_type_unregister (disk_type);
-}
-
 /**
  * Return the next disk type registers, after "type".  If "type" is
  * NULL, returns the first disk type.
@@ -153,7 +138,7 @@ ped_disk_type_get (const char* name)
 PedDiskType*
 ped_disk_probe (PedDevice* dev)
 {
-        PedDiskType *walk = NULL;
+        PedDiskType* walk = NULL;
 
         PED_ASSERT (dev != NULL, return NULL);
 
@@ -162,10 +147,9 @@ ped_disk_probe (PedDevice* dev)
 
         ped_exception_fetch_all ();
         for (walk = ped_disk_type_get_next (NULL); walk;
-             walk = ped_disk_type_get_next (walk)) {
+             walk = ped_disk_type_get_next (walk))
                 if (walk->ops->probe (dev))
                         break;
-        }
 
         if (ped_exception)
                 ped_exception_catch ();
@@ -176,12 +160,12 @@ ped_disk_probe (PedDevice* dev)
 }
 
 /**
- * Read the partition table off a device (if one is found). 
- * 
+ * Read the partition table off a device (if one is found).
+ *
  * \warning May modify \p dev->cylinders, \p dev->heads and \p dev->sectors
  *      if the partition table indicates that the existing values
  *      are incorrect.
- * 
+ *
  * \return A new \link _PedDisk PedDisk \endlink object;
  *         NULL on failure (e.g. partition table not detected).
  */
@@ -224,23 +208,24 @@ static int
 _add_duplicate_part (PedDisk* disk, PedPartition* old_part)
 {
 	PedPartition*	new_part;
-	PedConstraint*	constraint_exact;
+	int ret;
 
 	new_part = disk->type->ops->partition_duplicate (old_part);
 	if (!new_part)
 		goto error;
 	new_part->disk = disk;
 
-	constraint_exact = ped_constraint_exact (&new_part->geom);
-	if (!constraint_exact)
+	_disk_push_update_mode (disk);
+	ret = _disk_raw_add (disk, new_part);
+	_disk_pop_update_mode (disk);
+	if (!ret)
 		goto error_destroy_new_part;
-	if (!ped_disk_add_partition (disk, new_part, constraint_exact))
-       		goto error_destroy_constraint_exact;
-	ped_constraint_destroy (constraint_exact);
+#ifdef DEBUG
+	if (!_disk_check_sanity (disk))
+		goto error_destroy_new_part;
+#endif
 	return 1;
 
-error_destroy_constraint_exact:
-	ped_constraint_destroy (constraint_exact);
 error_destroy_new_part:
 	ped_partition_destroy (new_part);
 error:
@@ -288,9 +273,9 @@ error:
 /**
  * Remove all identifying signatures of a partition table,
  * except for partition tables of a given type.
- * 
+ *
  * \return 0 on error, 1 otherwise.
- * 
+ *
  * \sa ped_disk_clobber()
  */
 int
@@ -330,11 +315,11 @@ error:
 	return 0;
 }
 
-/** 
+/**
  * Remove all identifying signatures of a partition table,
- * 
+ *
  * \return 0 on error, 1 otherwise.
- * 
+ *
  * \sa ped_disk_clobber_exclude()
  */
 int
@@ -348,7 +333,7 @@ ped_disk_clobber (PedDevice* dev)
  *
  * This new partition table is only created in-memory, and nothing is written
  * to disk until ped_disk_commit_to_dev() is called.
- * 
+ *
  * \return The newly constructed \link _PedDisk PedDisk \endlink,
  *      NULL on failure.
  */
@@ -391,7 +376,7 @@ _ped_disk_alloc (const PedDevice* dev, const PedDiskType* disk_type)
 	disk->part_list = NULL;
 	return disk;
 
-	ped_free (disk);
+	free (disk);
 error:
 	return NULL;
 }
@@ -401,7 +386,7 @@ _ped_disk_free (PedDisk* disk)
 {
 	_disk_push_update_mode (disk);
 	ped_disk_delete_all (disk);
-	ped_free (disk);
+	free (disk);
 }
 
 /**
@@ -430,7 +415,7 @@ ped_disk_destroy (PedDisk* disk)
  * use the new blkpg interface to tell Linux where each partition
  * starts/ends, etc. In this case, Linux does not need to have support for
  * a specific type of partition table.
- * 
+ *
  * \return 0 on failure, 1 otherwise.
  */
 int
@@ -598,8 +583,8 @@ ped_disk_check (const PedDisk* disk)
 				  "%s."),
 				walk->num, part_size, fs_size);
 
-			ped_free (part_size);
-			ped_free (fs_size);
+			free (part_size);
+			free (fs_size);
 
 			if (choice != PED_EXCEPTION_IGNORE)
 				return 0;
@@ -644,7 +629,7 @@ ped_disk_get_primary_partition_count (const PedDisk* disk)
 }
 
 /**
- * Get the highest partition number on \p disk.
+ * Get the highest available partition number on \p disk.
  */
 int
 ped_disk_get_last_partition_num (const PedDisk* disk)
@@ -663,9 +648,23 @@ ped_disk_get_last_partition_num (const PedDisk* disk)
 	return highest;
 }
 
+/**
+ * Get the highest supported partition number on \p disk.
+ *
+ * \return 0 if call fails. 1 otherwise.
+ */
+bool
+ped_disk_get_max_supported_partition_count(const PedDisk* disk, int* supported)
+{
+	PED_ASSERT(disk != NULL, return -1);
+	PED_ASSERT(disk->type->ops->get_max_supported_partition_count != NULL, return -1);
+
+	return disk->type->ops->get_max_supported_partition_count(disk, supported);
+}
+
 /**
  * Get the maximum number of (primary) partitions the disk label supports.
- * 
+ *
  * For example, MacIntosh partition maps can have different sizes,
  * and accordingly support a different number of partitions.
  */
@@ -836,7 +835,7 @@ _alloc_extended_freespace (PedDisk* disk)
 
 	last_end = extended_part->geom.start;
 	last = NULL;
-	
+
 	for (walk = extended_part->part_list; walk; walk = walk->next) {
 		if (walk->geom.start > last_end + 1) {
 			free_space = ped_partition_new (
@@ -966,7 +965,7 @@ _disk_pop_update_mode (PedDisk* disk)
  * \addtogroup PedPartition
  *
  * \brief Partition access.
- * 
+ *
  * @{
  */
 
@@ -998,7 +997,7 @@ _ped_partition_alloc (const PedDisk* disk, PedPartitionType type,
 	return part;
 
 error_free_part:
-	ped_free (part);
+	free (part);
 error:
 	return NULL;
 }
@@ -1006,7 +1005,7 @@ error:
 void
 _ped_partition_free (PedPartition* part)
 {
-	ped_free (part);
+	free (part);
 }
 
 int
@@ -1044,13 +1043,13 @@ fail:
  *
  * \note The constructed partition is not added to <tt>disk</tt>'s
  *      partition table. Use ped_disk_add_partition() to do this.
- * 
+ *
  * \return A new \link _PedPartition PedPartition \endlink object,
  *      NULL on failure.
  *
  * \throws PED_EXCEPTION_ERROR if \p type is \p EXTENDED or \p LOGICAL but the
  *      label does not support this concept.
- */ 
+ */
 PedPartition*
 ped_partition_new (const PedDisk* disk, PedPartitionType type,
 		   const PedFileSystemType* fs_type, PedSector start,
@@ -1127,7 +1126,7 @@ ped_partition_is_active (const PedPartition* part)
 
 /**
  * Set the state (\c 1 or \c 0) of a flag on a partition.
- * 
+ *
  * Flags are disk label specific, although they have a global
  * "namespace": the flag PED_PARTITION_BOOT, for example, roughly means
  * "this" partition is bootable". But this means different things on different
@@ -1135,7 +1134,7 @@ ped_partition_is_active (const PedPartition* part)
  * on MS-DOS disk labels, there can only be one boot partition, and this
  * refers to the partition that will be booted from on startup. On PC98
  * disk labels, the user can choose from any bootable partition on startup.
- * 
+ *
  * \note It is an error to call this on an unavailable flag -- use
  * ped_partition_is_flag_available() to determine which flags are available
  * for a given disk label.
@@ -1254,7 +1253,7 @@ _assert_partition_name_feature (const PedDiskType* disk_type)
  * ped_disk_type_check_feature (part->disk->type, PED_DISK_TYPE_PARTITION_NAME);
  *      \endcode
  *      to check whether this feature is enabled for a label.
- *      
+ *
  * \note \p name will not be modified by libparted. It can be freed
  *      by the caller immediately after ped_partition_set_name() is called.
  *
@@ -1321,7 +1320,7 @@ ped_disk_extended_partition (const PedDisk* disk)
 	return walk;
 }
 
-/** 
+/**
  * Return the next partition after \p part on \p disk. If \p part is \c NULL,
  * return the first partition. If \p part is the last partition, returns
  * \c NULL. If \p part is an extended partition, returns the first logical
@@ -1375,7 +1374,7 @@ _disk_check_sanity (PedDisk* disk)
 #endif
 
 /**
- * Returns the partition numbered \p num. 
+ * Returns the partition numbered \p num.
  *
  * \return \c NULL if the specified partition does not exist.
  */
@@ -1517,7 +1516,7 @@ _disk_raw_add (PedDisk* disk, PedPartition* part)
 				ext_part->part_list = part;
 			else
 				disk->part_list = part;
-		} 
+		}
 	}
 
 	return 1;
@@ -1536,7 +1535,7 @@ _partition_get_overlap_constraint (PedPartition* part, PedGeometry* geom)
 
 	if (part->type & PED_PARTITION_LOGICAL) {
 		PedPartition* ext_part;
-		
+
 		ext_part = ped_disk_extended_partition (part->disk);
 		PED_ASSERT (ext_part != NULL, return NULL);
 
@@ -1575,7 +1574,7 @@ _partition_get_overlap_constraint (PedPartition* part, PedGeometry* geom)
  * Returns \c 0 if the partition, \p part overlaps with any partitions on the
  * \p disk.  The geometry of \p part is taken to be \p geom, NOT \p part->geom
  * (the idea here is to check if \p geom is valid, before changing \p part).
- * 
+ *
  * This is useful for seeing if a resized partitions new geometry is going to
  * fit, without the existing geomtry getting in the way.
  *
@@ -1694,6 +1693,31 @@ _check_extended_partition (PedDisk* disk, PedPartition* part)
 	return 1;
 }
 
+static PedSector
+_partition_max_start (char const *label_type)
+{
+  /* List partition table names (a la disk->type->name) for which
+     the partition length, in sectors, must fit in 32 bytes.  */
+  static char const *const max_32[] = {"msdos", "dvh"};
+  unsigned int i;
+
+  for (i = 0; i < sizeof max_32 / sizeof *max_32; i++)
+    if (strcmp (label_type, max_32[i]) == 0)
+      return UINT32_MAX;
+
+  return TYPE_MAXIMUM (PedSector);
+}
+
+static PedSector
+_partition_max_len (char const *label_type)
+{
+  /* NOTE: for now, they happen to be the same, so don't
+     duplicate needlessly.  Of course, if there's some format
+     with different length and starting sector limits, then
+     these functions will diverge.  */
+  return _partition_max_start (label_type);
+}
+
 static int
 _check_partition (PedDisk* disk, PedPartition* part)
 {
@@ -1734,12 +1758,51 @@ _check_partition (PedDisk* disk, PedPartition* part)
 		return 0;
 	}
 
+	if (!(part->type & PED_PARTITION_METADATA)) {
+		char const *label_type = disk->type->name;
+		/* Enforce some restrictions inherent in the DOS
+		   partition table format.  Without these, one would be able
+		   to create a 2TB partition (or larger), and it would work,
+		   but only until the next reboot.  This was insidious: the
+		   too-large partition would work initially, because with
+		   Linux-2.4.x and newer we set the partition start sector
+		   and length (in sectors) accurately and directly via the
+		   BLKPG ioctl.  However, only the last 32 bits of each
+		   number would be written to the partition table, and the
+		   next time the system would read/use those corrupted numbers
+		   it would usually complain about an invalid partition.
+                   The same applies to the starting sector number.  */
+
+		if (part->geom.length > _partition_max_len (label_type)) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+				_("partition length of %jd sectors exceeds the "
+                                  "%s-partition-table-imposed maximum of %jd"),
+				part->geom.length,
+                                label_type,
+                                _partition_max_len (label_type));
+			return 0;
+		}
+
+		/* The starting sector number must fit in 32 bytes.  */
+		if (part->geom.start > _partition_max_start (label_type)) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+				_("starting sector number, %jd exceeds the"
+                                  " %s-partition-table-imposed maximum of %jd"),
+                                part->geom.start,
+                                label_type,
+                                _partition_max_start (label_type));
+			return 0;
+		}
+	}
+
 	return 1;
 }
 
 /**
  * Adds PedPartition \p part to PedPartition \p disk.
- * 
+ *
  * \warning The partition's geometry may be changed, subject to \p constraint.
  * You could set \p constraint to <tt>ped_constraint_exact(&part->geom)</tt>,
  * but many partition table schemes have special requirements on the start
@@ -1747,7 +1810,7 @@ _check_partition (PedDisk* disk, PedPartition* part)
  * will probably mean that this function will fail (in which
  * case \p part will be left unmodified)
  * \p part is assigned a number (\p part->num) in this process.
- * 
+ *
  * \return \c 0 on failure.
  */
 int
@@ -1911,7 +1974,7 @@ ped_disk_delete_all (PedDisk* disk)
  * to \p constraint.
  *
  * \warning The constraint warning from ped_disk_add_partition() applies.
- * 
+ *
  * \note this function does not modify the contents of the partition.  You need
  *       to call ped_file_system_resize() separately.
  */
@@ -1972,7 +2035,7 @@ error_pop_update_mode:
 /**
  * Grow PedPartition \p part geometry to the maximum possible subject to
  * \p constraint.  The new geometry will be a superset of the old geometry.
- * 
+ *
  * \return 0 on failure
  */
 int
@@ -2066,7 +2129,7 @@ ped_disk_get_max_partition_geometry (PedDisk* disk, PedPartition* part,
 	return max_geom;
 }
 
-/** 
+/**
  * Reduce the size of the extended partition to a minimum while still wrapping
  * its logical partitions.  If there are no logical partitions, remove the
  * extended partition.
@@ -2122,11 +2185,11 @@ ped_disk_minimize_extended_partition (PedDisk* disk)
 
 /**
  * Returns a name that seems mildly appropriate for a partition type \p type.
- * 
+ *
  * Eg, if you pass (PED_PARTITION_LOGICAL & PED_PARTITION_FREESPACE), it
  * will return "free".  This isn't to be taken too seriously - it's just
  * useful for user interfaces, so you can show the user something ;-)
- * 
+ *
  * \note The returned string will be in English.  However,
  * translations are provided, so the caller can call
  * dgettext("parted", RESULT) on the result.
@@ -2150,7 +2213,7 @@ ped_partition_type_get_name (PedPartitionType type)
 
 /**
  * Returns a name for a \p flag, e.g. PED_PARTITION_BOOT will return "boot".
- * 
+ *
  * \note The returned string will be in English.  However,
  * translations are provided, so the caller can call
  * dgettext("parted", RESULT) on the result.
@@ -2161,6 +2224,8 @@ ped_partition_flag_get_name (PedPartitionFlag flag)
 	switch (flag) {
 	case PED_PARTITION_BOOT:
 		return N_("boot");
+	case PED_PARTITION_BIOS_GRUB:
+		return N_("bios_grub");
 	case PED_PARTITION_ROOT:
 		return N_("root");
 	case PED_PARTITION_SWAP:
@@ -2193,8 +2258,8 @@ ped_partition_flag_get_name (PedPartitionFlag flag)
 }
 
 /**
- * Iterates through all flags. 
- * 
+ * Iterates through all flags.
+ *
  * ped_partition_flag_next(0) returns the first flag
  *
  * \return the next flag, or 0 if there are no more flags
@@ -2206,7 +2271,7 @@ ped_partition_flag_next (PedPartitionFlag flag)
 }
 
 /**
- * Returns the flag associated with \p name.  
+ * Returns the flag associated with \p name.
  *
  * \p name can be the English
  * string, or the translation for the native language.
