@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2007-2008 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /** \file exception.c */
@@ -23,7 +22,7 @@
  * \addtogroup PedException
  *
  * \brief Exception handling.
- * 
+ *
  * There are a few types of exceptions: PED_EXCEPTION_INFORMATION,
  * PED_EXCEPTION_WARNING, PED_EXCEPTION_ERROR, PED_EXCEPTION_FATAL,
  * PED_EXCEPTION_BUG.
@@ -74,7 +73,7 @@ static PedExceptionHandler*	ex_handler = default_handler;
 static PedException*		ex = NULL;
 static int			ex_fetch_count = 0;
 
-static char*	type_strings [] = {
+static const char *const type_strings [] = {
 	N_("Information"),
 	N_("Warning"),
 	N_("Error"),
@@ -83,7 +82,7 @@ static char*	type_strings [] = {
 	N_("No Implementation")
 };
 
-static char*	option_strings [] = {
+static const char *const option_strings [] = {
 	N_("Fix"),
 	N_("Yes"),
 	N_("No"),
@@ -99,7 +98,7 @@ static char*	option_strings [] = {
 char*
 ped_exception_get_type_string (PedExceptionType ex_type)
 {
-	return type_strings [ex_type - 1];
+	return (char *) type_strings [ex_type - 1];
 }
 
 /* FIXME: move this out to the prospective math.c */
@@ -122,7 +121,7 @@ ped_log2 (int n)
 char*
 ped_exception_get_option_string (PedExceptionOption ex_opt)
 {
-	return option_strings [ped_log2 (ex_opt)];
+	return (char *) option_strings [ped_log2 (ex_opt)];
 }
 
 static PedExceptionOption
@@ -187,13 +186,12 @@ ped_exception_get_handler (void)
 void
 ped_exception_catch ()
 {
-	if (ped_exception) {
-		ped_exception = 0;
-
-		ped_free (ex->message);
-		ped_free (ex);
-		ex = NULL;
-	}
+        if (ped_exception) {
+                ped_exception = 0;
+                free (ex->message);
+                free (ex);
+                ex = NULL;
+        }
 }
 
 static PedExceptionOption
@@ -216,7 +214,7 @@ do_throw ()
  * Throw an exception.
  *
  * You can also use this in a program using libparted.
- * "message" is a printf-like format string, so you can do 
+ * "message" is a printf-like format string, so you can do
  *
  * \code
  * ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_RETRY_CANCEL,
@@ -244,8 +242,8 @@ ped_exception_throw (PedExceptionType ex_type,
 	ex->type = ex_type;
 	ex->options = ex_opts;
 
-	while (1) {
-			ex->message = (char*) malloc (size);
+	while (message) {
+			ex->message = (char*) malloc (size * sizeof (char));
 			if (!ex->message)
 					goto no_memory;
 
@@ -257,6 +255,7 @@ ped_exception_throw (PedExceptionType ex_type,
 					break;
 
 			size += 10;
+			free (ex->message);
 	}
 
 	return do_throw ();
