@@ -132,7 +132,7 @@ int ext2_move_blocks(struct ext2_fs *fs, blk_t src, blk_t num, blk_t dest)
 		if (!ext2_read_blocks(fs, buf, src, num)) return 0;
 		if (!ext2_write_blocks(fs, buf, dest, num)) return 0;
 
-		ped_free(buf);
+		free(buf);
 		return 1;
 	}
 	ped_exception_catch();
@@ -212,7 +212,7 @@ int ext2_zero_blocks(struct ext2_fs *fs, blk_t block, blk_t num)
 			goto error_free_buf;
 		if (!ext2_write_blocks(fs, buf, block, num))
 			goto error_free_buf;
-		ped_free(buf);
+		free(buf);
 		return 1;
 	}
 	ped_exception_catch();
@@ -232,7 +232,7 @@ int ext2_zero_blocks(struct ext2_fs *fs, blk_t block, blk_t num)
 				goto error_free_buf;
 		}
 
-		ped_free(buf);
+		free(buf);
 		return 1;
 	}
 	ped_exception_catch();
@@ -252,7 +252,7 @@ int ext2_zero_blocks(struct ext2_fs *fs, blk_t block, blk_t num)
 	return 1;
 
 error_free_buf:
-	ped_free(buf);
+	free(buf);
 error:
 	return 0;
 }
@@ -569,7 +569,7 @@ static int is_root(int x, int y)
 /* check whether group contains a superblock copy on file systems
  * where not all groups have one (sparse superblock feature) */
 int ext2_is_group_sparse(struct ext2_fs *fs, int group)
-{       
+{
 	if (!fs->sparse)
 		return 1;
 
@@ -588,8 +588,8 @@ void ext2_close(struct ext2_fs *fs)
 
 	fs->devhandle->ops->close(fs->devhandle->cookie);
 
-	ped_free(fs->gd);
-	ped_free(fs);
+	free(fs->gd);
+	free(fs);
 }
 
 int ext2_commit_metadata(struct ext2_fs *fs, int copies)
@@ -727,7 +727,10 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 			    EXT2_FEATURE_RO_COMPAT_LARGE_FILE)))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-		     _("File system has an incompatible feature enabled."));
+		     _("File system has an incompatible feature enabled.  "
+		       "Compatible features are has_journal, dir_index, "
+		       "filetype, sparse_super and large_file.  "
+		       "Use tune2fs or debugfs to remove features."));
 		goto error_free_fs;
 	}
 
@@ -776,15 +779,15 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 
 	ext2_read_blocks(fs, fs->gd, EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb) + 1,
 			 fs->gdblocks);
-                
+
 	fs->metadirty = 0;
 	return fs;
 
-	ped_free(fs->gd);
+	free(fs->gd);
 error_deinit_bcache:
 	ext2_bcache_deinit(fs);
 error_free_fs:
-	ped_free(fs);
+	free(fs);
 error:
 	return NULL;
 }
