@@ -51,11 +51,11 @@ struct ext2_block_entry
  * 	and the first data block in the group.  This can change when a
  * 	filesystem is resized, because the size of the group descriptors is
  * 	proportional to the size of the filesystem.
- * 
+ *
  * 	- allocentries is the size of the "block" array.  It is a tuneable
  * 	parameter that determines how many blocks can be moved in each
  * 	pass.
- * 
+ *
  * 	- usedentries says how many entries of the "block" array have been
  * 	used.  That is, how many blocks have been scheduled so far to
  * 	be moved.
@@ -492,7 +492,7 @@ static int ext2_block_relocator_copy(struct ext2_fs *fs, struct ext2_block_reloc
 			}
 		}
 
-		ped_free(buf);
+		free(buf);
 
 		if (fs->opt_safe)
 			ext2_sync(fs);
@@ -520,7 +520,7 @@ static int ext2_block_relocator_copy(struct ext2_fs *fs, struct ext2_block_reloc
 	return 1;
 
 error_free_buf:
-	ped_free(buf);
+	free(buf);
 error:
 	return 0;
 }
@@ -691,10 +691,11 @@ static int ext2_block_relocator_flush(struct ext2_fs *fs, struct ext2_block_relo
 		{
 			/* FIXXXME gross hack */
 			fprintf(stderr, "relocating %s blocks",
-				((char *[4]){"direct",
-						     "singly indirect",
-						     "doubly indirect",
-						     "triply indirect"})[i]);
+				((const char *const [4])
+				 {"direct",
+				  "singly indirect",
+				  "doubly indirect",
+				  "triply indirect"})[i]);
 			fflush(stderr);
 		}
 
