@@ -1,10 +1,10 @@
 /*
     libparted
-    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1998-2001, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,15 +13,12 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef FAT_H_INCLUDED
 #define FAT_H_INCLUDED
 
-#include <config.h>
-
 #include <parted/parted.h>
 #include <parted/endian.h>
 #include <parted/debug.h>
@@ -138,7 +135,7 @@ struct _FatSpecific {
 #define DIRECTORY_ATTR		0x10
 #define ARCH_ATTR		0x20
 
-#define MAX_FAT12_CLUSTERS	4086      
+#define MAX_FAT12_CLUSTERS	4086
 #define MAX_FAT16_CLUSTERS	65526
 #define MAX_FAT32_CLUSTERS	2000000
 
