@@ -1,10 +1,10 @@
 /*
     libparted
-    Copyright (C) 1998, 1999, 2000, 2005, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1998-2000, 2005, 2007-2008 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,16 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
-
-    I can also be contacted at:
-
-    Andrew Clausen
-    18 Shaw St
-    Ashwood, 3147
-    Victoria, Australia
-
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -51,7 +42,7 @@ fat_traverse_entries_per_buffer (FatTraverseInfo* trav_info)
 static int
 is_last_buffer (FatTraverseInfo* trav_info) {
 	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
-    
+
 	if (trav_info->is_legacy_root_dir)
 		return 1;
 	else
@@ -62,7 +53,7 @@ static int
 write_root_dir (FatTraverseInfo* trav_info)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
-	
+
 	if (!ped_geometry_write (trav_info->fs->geom, trav_info->dir_entries,
 				 fs_info->root_dir_offset,
 				 fs_info->root_dir_sector_count))
@@ -129,7 +120,7 @@ fat_traverse_mark_dirty (FatTraverseInfo* trav_info)
 
 FatTraverseInfo*
 fat_traverse_begin (PedFileSystem* fs, FatCluster start_cluster,
-		    char* dir_name)
+		    const char* dir_name)
 {
 	FatSpecific*		fs_info = FAT_SPECIFIC (fs);
 	FatTraverseInfo*	trav_info;
@@ -174,11 +165,11 @@ fat_traverse_begin (PedFileSystem* fs, FatCluster start_cluster,
 	return trav_info;
 
 error_free_dir_entries:
-	ped_free (trav_info->dir_entries);
+	free (trav_info->dir_entries);
 error_free_dir_name:
-	ped_free (trav_info->dir_name);
+	free (trav_info->dir_name);
 error_free_trav_info:
-	ped_free (trav_info);
+	free (trav_info);
 error:
 	return NULL;
 }
@@ -190,9 +181,9 @@ fat_traverse_complete (FatTraverseInfo* trav_info)
 		if (!write_dir_buffer (trav_info))
 			return 0;
 	}
-	ped_free (trav_info->dir_entries);
-	ped_free (trav_info->dir_name);
-	ped_free (trav_info);
+	free (trav_info->dir_entries);
+	free (trav_info->dir_name);
+	free (trav_info);
 	return 1;
 }
 
@@ -267,7 +258,7 @@ fat_dir_entry_set_first_cluster (FatDirEntry* dir_entry, PedFileSystem* fs,
 		case FAT_TYPE_FAT16:
 		dir_entry->first_cluster = PED_CPU_TO_LE16 (cluster);
 		break;
-            
+
 		case FAT_TYPE_FAT32:
 		dir_entry->first_cluster
 			= PED_CPU_TO_LE16 (cluster & 0xffff);
@@ -332,7 +323,7 @@ fat_dir_entry_has_first_cluster (FatDirEntry* dir_entry, PedFileSystem* fs)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
 	FatCluster	first_cluster;
-    
+
 	if (!fat_dir_entry_is_file (dir_entry)
 		&& !fat_dir_entry_is_directory (dir_entry))
 		return 0;
@@ -349,22 +340,24 @@ fat_dir_entry_has_first_cluster (FatDirEntry* dir_entry, PedFileSystem* fs)
     decrypts silly DOS names to FILENAME.EXT
 */
 void
-fat_dir_entry_get_name (FatDirEntry*dir_entry, char *result) {
-	int     i;
-	char   *src;
+fat_dir_entry_get_name (const FatDirEntry *dir_entry, char *result) {
+	size_t i;
+	const char *src;
+	const char *ext;
 
 	src = dir_entry->name;
 
-	for (i=0; i<8; i++) {
+	for (i=0; i < sizeof dir_entry->name; i++) {
 		if (src[i] == ' ' || src[i] == 0) break;
 		*result++ = src[i];
 	}
 
-	if (src[8] != ' ' && src[8] != 0) {
+	ext = (const char *) dir_entry->extension;
+	if (ext[0] != ' ' && ext[0] != 0) {
 		*result++ = '.';
-		for (i=8; i<11; i++) {
-			if (src[i] == ' ' || src[i] == 0) break;
-			*result++ = src[i];
+		for (i=0; i < sizeof dir_entry->extension; i++) {
+			if (ext[i] == ' ' || ext[i] == 0) break;
+			*result++ = ext[i];
 		}
 	}
 
