@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+    Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef DISCOVER_ONLY
@@ -49,7 +48,7 @@ hfsplus_extent_key_cmp(HfsPPrivateGenericKey* a, HfsPPrivateGenericKey* b)
 
 	if (key1->file_ID != key2->file_ID)
 		return PED_BE32_TO_CPU(key1->file_ID) <
-		       PED_BE32_TO_CPU(key2->file_ID) ? 
+		       PED_BE32_TO_CPU(key2->file_ID) ?
 				-1 : +1;
 
 	if (key1->type != key2->type)
@@ -57,7 +56,7 @@ hfsplus_extent_key_cmp(HfsPPrivateGenericKey* a, HfsPPrivateGenericKey* b)
 
 	if (key1->start == key2->start)
 		return 0;
-	return PED_BE32_TO_CPU(key1->start) < 
+	return PED_BE32_TO_CPU(key1->start) <
 	       PED_BE32_TO_CPU(key2->start) ?
 			-1 : +1;
 }
@@ -120,13 +119,13 @@ hfsplus_btree_search (HfsPPrivateFile* b_tree_file, HfsPPrivateGenericKey* key,
 					PED_EXCEPTION_ERROR,
 					PED_EXCEPTION_CANCEL,
 					_("The file system contains errors."));
-				ped_free (node);
+				free (node);
 				return 0;
 			}
 			if (hfsplus_extent_key_cmp(record_key, key) <= 0)
 				break;
 		}
-		if (!i) { ped_free (node); return 0; }
+		if (!i) { free (node); return 0; }
 		if (desc->type == HFS_IDX_NODE) {
 			unsigned int 	skip;
 
@@ -137,10 +136,10 @@ hfsplus_btree_search (HfsPPrivateFile* b_tree_file, HfsPPrivateGenericKey* key,
 			if (!hfsplus_file_read(b_tree_file, node,
 					       (PedSector) node_number * size,
 					       size)) {
-				ped_free (node);
+				free (node);
 				return 0;
 			}
-		} else 
+		} else
 			break;
 	}
 
@@ -157,7 +156,7 @@ hfsplus_btree_search (HfsPPrivateFile* b_tree_file, HfsPPrivateGenericKey* key,
 	}
 
 	/* success */
-	ped_free (node);
+	free (node);
 	return 1;
 }
 
@@ -169,7 +168,7 @@ hfsplus_free_bad_blocks_list(HfsPPrivateLinkExtent* first)
 
 	while (first) {
 		next = first->next;
-		ped_free (first);
+		free (first);
 		first = next;
 	}
 }
@@ -258,7 +257,7 @@ hfsplus_is_bad_block (const PedFileSystem *fs, unsigned int fblock)
 	for (walk = priv_data->bad_blocks_xtent_list; walk; walk = walk->next) {
 		/* Won't compile without the strange cast ! gcc bug ? */
 		/* or maybe C subtilties... */
-		if ((fblock >= PED_BE32_TO_CPU (walk->extent.start_block)) && 
+		if ((fblock >= PED_BE32_TO_CPU (walk->extent.start_block)) &&
 		    (fblock <  (unsigned int)(PED_BE32_TO_CPU (
 						walk->extent.start_block)
 			       + PED_BE32_TO_CPU (walk->extent.block_count))))
@@ -335,9 +334,9 @@ hfsplus_get_min_size (const PedFileSystem *fs)
 		hfs_sect_block =
 		    PED_BE32_TO_CPU (hfs_priv_data->mdb->block_size)
 		    / PED_SECTOR_SIZE_DEFAULT;
-		/* 
+		/*
 		 * if hfs+ is embedded in an hfs wrapper then the new size is :
-		 * the new size of the hfs+ volume rounded up to the size 
+		 * the new size of the hfs+ volume rounded up to the size
 		 *     of hfs blocks
 		 * + the minimum size of the hfs wrapper without any hfs+
 		 *     modification
