@@ -1,7 +1,7 @@
 /* -*- Mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2001, 2007 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2001, 2007, 2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <parted/parted.h>
 #include <parted/debug.h>
 #include <parted/endian.h>
+#include <stdbool.h>
 
 #if ENABLE_NLS
 #  include <libintl.h>
@@ -33,6 +34,7 @@
 #endif /* ENABLE_NLS */
 
 #define	AIX_LABEL_MAGIC		0xc9c2d4c1
+#define	MAX_TOTAL_PART		16
 
 static PedDiskType aix_disk_type;
 
@@ -57,7 +59,7 @@ read_sector (const PedDevice *dev, char **buf)
 	char *b = ped_malloc (dev->sector_size);
 	PED_ASSERT (b != NULL, return 0);
 	if (!ped_device_read (dev, b, 0, 1)) {
-		ped_free (b);
+		free (b);
 		return 0;
 	}
 	*buf = b;
@@ -73,7 +75,7 @@ aix_probe (const PedDevice *dev)
 	if (!read_sector (dev, &label))
 		return 0;
 	unsigned int magic = aix_label_magic_get (label);
-	ped_free (label);
+	free (label);
 	return magic == AIX_LABEL_MAGIC;
 }
 
@@ -92,7 +94,7 @@ aix_clobber (PedDevice* dev)
 
 	aix_label_magic_set (label, 0);
 	int result = ped_device_write (dev, label, 0, 1);
-	ped_free (label);
+	free (label);
 	return result;
 }
 #endif /* !DISCOVER_ONLY */
@@ -113,7 +115,7 @@ static PedDisk*
 aix_duplicate (const PedDisk* disk)
 {
 	PedDisk*	new_disk;
-       
+
 	new_disk = ped_disk_new_fresh (disk->dev, &aix_disk_type);
 	if (!new_disk)
 		return NULL;
@@ -221,6 +223,13 @@ aix_get_max_primary_partition_count (const PedDisk* disk)
 	return 4;
 }
 
+static bool
+aix_get_max_supported_partition_count (const PedDisk* disk, int *max_n)
+{
+	*max_n = MAX_TOTAL_PART;
+	return true;
+}
+
 static int
 aix_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
@@ -270,6 +279,8 @@ static PedDiskOps aix_disk_ops = {
 	alloc_metadata:		aix_alloc_metadata,
 	get_max_primary_partition_count:
 				aix_get_max_primary_partition_count,
+	get_max_supported_partition_count:
+				aix_get_max_supported_partition_count,
 
 	partition_set_name:		NULL,
 	partition_get_name:		NULL,
