@@ -1,11 +1,11 @@
 /* -*- Mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2001, 2007 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2001, 2007-2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
     Contributor:  Phil Knirsch <phil@redhat.de>
                   Harald Hoyer <harald@redhat.de>
@@ -29,6 +28,7 @@
 #include <time.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <stdbool.h>
 
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -38,7 +38,7 @@
 
 #include <parted/vtoc.h>
 #include <parted/fdasd.h>
-#include <parted/linux.h>
+#include <arch/linux.h>
 
 #include <libintl.h>
 #if ENABLE_NLS
@@ -47,6 +47,8 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
+#include "misc.h"
+
 #define PARTITION_LINUX_SWAP 0x82
 #define PARTITION_LINUX 0x83
 #define PARTITION_LINUX_EXT 0x85
@@ -127,6 +129,7 @@ static PedDiskOps dasd_disk_ops = {
 
 	alloc_metadata: dasd_alloc_metadata,
 	get_max_primary_partition_count: dasd_get_max_primary_partition_count,
+	get_max_supported_partition_count: dasd_get_max_supported_partition_count,
 
 	partition_duplicate: NULL
 };
@@ -154,7 +157,7 @@ dasd_alloc (const PedDevice* dev)
 
 	disk->disk_specific = disk_specific = ped_malloc(sizeof(DasdDiskSpecific));
 	if (!disk->disk_specific) {
-		ped_free (disk);
+		free (disk);
 		return NULL;
 	}
 
@@ -165,8 +168,8 @@ dasd_alloc (const PedDevice* dev)
 		ped_exception_throw(PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 							_("Unable to determine the block "
 							  "size of this dasd"));
-		ped_free(disk_specific);
-		ped_free(disk);
+		free(disk_specific);
+		free(disk);
 		return NULL;
 	}
 
@@ -212,13 +215,13 @@ ped_disk_dasd_done ()
 static int
 dasd_probe (const PedDevice *dev)
 {
-	char *errstr = 0;
 	LinuxSpecific* arch_specific;
 	struct fdasd_anchor anchor;
 
 	PED_ASSERT(dev != NULL, return 0);
-	PED_ASSERT((dev->type == PED_DEVICE_DASD
-			   || dev->type == PED_DEVICE_VIODASD), return 0);
+
+	if (!(dev->type == PED_DEVICE_DASD || dev->type == PED_DEVICE_VIODASD))
+		return 0;
 
 	arch_specific = LINUX_SPECIFIC(dev);
 
@@ -236,9 +239,10 @@ dasd_probe (const PedDevice *dev)
 
 	return 1;
 
-error_cleanup:
+ error_cleanup:
 	fdasd_cleanup(&anchor);
-	ped_exception_throw(PED_EXCEPTION_ERROR,PED_EXCEPTION_IGNORE_CANCEL,errstr);
+	ped_exception_throw(PED_EXCEPTION_ERROR,PED_EXCEPTION_IGNORE_CANCEL,
+			    "Error while probing device %s.", dev->path);
 
 	return 0;
 }
@@ -269,6 +273,7 @@ dasd_read (PedDisk* disk)
 	char str[20];
 	PedDevice* dev;
 	PedPartition* part;
+	PedFileSystemType *fs;
 	PedSector start, end;
 	PedConstraint* constraint_exact;
 	partition_info_t *p;
@@ -358,7 +363,8 @@ dasd_read (PedDisk* disk)
 				* (long long) disk->dev->hw_geom.sectors
 				* (long long) disk_specific->real_sector_size
 				/ (long long) disk->dev->sector_size - 1;
-		part = ped_partition_new(disk, 0, NULL, start, end);
+		part = ped_partition_new(disk, PED_PARTITION_NORMAL, NULL,
+                                         start, end);
         PDEBUG;
 
 		if (!part)
@@ -379,19 +385,24 @@ dasd_read (PedDisk* disk)
 
 		dasd_data = part->disk_specific;
 
-		if (strncmp(PART_TYPE_RAID, str, 6) == 0)
+		if ((strncmp(PART_TYPE_RAID, str, 6) == 0) &&
+		    (ped_file_system_probe(&part->geom) == NULL))
 			ped_partition_set_flag(part, PED_PARTITION_RAID, 1);
 		else
 			ped_partition_set_flag(part, PED_PARTITION_RAID, 0);
 
-		if (strncmp(PART_TYPE_LVM, str, 6) == 0)
+		if ((strncmp(PART_TYPE_LVM, str, 6) == 0) &&
+		    (ped_file_system_probe(&part->geom) == NULL))
 			ped_partition_set_flag(part, PED_PARTITION_LVM, 1);
 		else
 			ped_partition_set_flag(part, PED_PARTITION_LVM, 0);
 
 		if (strncmp(PART_TYPE_SWAP, str, 6) == 0) {
-			dasd_data->system = PARTITION_LINUX_SWAP;
-			PDEBUG;
+			fs = ped_file_system_probe(&part->geom);
+			if (is_linux_swap(fs->name)) {
+				dasd_data->system = PARTITION_LINUX_SWAP;
+				PDEBUG;
+			}
 		}
 
 		vtoc_ebcdic_enc(p->f1->DS1DSNAM, p->f1->DS1DSNAM, 44);
@@ -415,7 +426,8 @@ dasd_read (PedDisk* disk)
 					* (long long) disk->dev->hw_geom.sectors
 					* (long long) disk_specific->real_sector_size
 					/ (long long) disk->dev->sector_size - 1;
-			part = ped_partition_new (disk, 0, NULL, start, end);
+			part = ped_partition_new (disk, PED_PARTITION_NORMAL,
+                                                  NULL, start, end);
 
 			if (!part)
 				goto error_close_dev;
@@ -627,8 +639,8 @@ dasd_partition_destroy (PedPartition* part)
 	PED_ASSERT(part != NULL, return);
 
 	if (ped_partition_is_active(part))
-		ped_free(part->disk_specific);
-	ped_free(part);
+		free(part->disk_specific);
+	free(part);
 }
 
 static int
@@ -703,6 +715,13 @@ dasd_get_max_primary_partition_count (const PedDisk* disk)
 	return USABLE_PARTITIONS;
 }
 
+static bool
+dasd_get_max_supported_partition_count (const PedDisk* disk, int *max_n)
+{
+	*max_n = dasd_get_max_primary_partition_count(disk);
+	return true;
+}
+
 static PedConstraint*
 _primary_constraint (PedDisk* disk)
 {
@@ -809,7 +828,7 @@ dasd_partition_set_system (PedPartition* part,
 	if (!fs_type) {
 		dasd_data->system = PARTITION_LINUX;
         PDEBUG;
-	} else if (!strcmp (fs_type->name, "linux-swap")) {
+	} else if (is_linux_swap (fs_type->name)) {
 		dasd_data->system = PARTITION_LINUX_SWAP;
         PDEBUG;
 	} else {
