@@ -4,8 +4,7 @@
     original version by Matt Domsch <Matt_Domsch@dell.com>
     Disclaimed into the Public Domain
 
-    Portions Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007
-        Free Software Foundation, Inc.
+    Portions Copyright (C) 2001-2003, 2005-2009 Free Software Foundation, Inc.
 
     EFI GUID Partition Table handling
     Per Intel EFI Specification v1.02
@@ -13,7 +12,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -22,8 +21,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -39,6 +37,8 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <uuid/uuid.h>
+#include <stdbool.h>
+#include "xalloc.h"
 
 #if ENABLE_NLS
 #  include <libintl.h>
@@ -87,6 +87,10 @@ typedef struct {
     ((efi_guid_t) { PED_CPU_TO_LE32 (0xC12A7328), PED_CPU_TO_LE16 (0xF81F), \
     		    PED_CPU_TO_LE16 (0x11d2), 0xBA, 0x4B, \
 		    { 0x00, 0xA0, 0xC9, 0x3E, 0xC9, 0x3B }})
+#define PARTITION_BIOS_GRUB_GUID \
+    ((efi_guid_t) { PED_CPU_TO_LE32 (0x21686148), PED_CPU_TO_LE16 (0x6449), \
+    		    PED_CPU_TO_LE16 (0x6E6f), 0x74, 0x4E, \
+		    { 0x65, 0x65, 0x64, 0x45, 0x46, 0x49 }})
 #define LEGACY_MBR_PARTITION_GUID \
     ((efi_guid_t) { PED_CPU_TO_LE32 (0x024DEE41), PED_CPU_TO_LE16 (0x33E7), \
 		    PED_CPU_TO_LE16 (0x11d3, 0x9D, 0x69, \
@@ -169,9 +173,9 @@ struct __attribute__ ((packed)) _GuidPartitionEntry_t {
 #define GPT_PMBR_SECTORS 1
 #define GPT_PRIMARY_HEADER_LBA 1
 #define GPT_HEADER_SECTORS 1
-#define GPT_PRIMARY_PART_TABLE_LBA 2 
+#define GPT_PRIMARY_PART_TABLE_LBA 2
 
-/* 
+/*
    These values are only defaults.  The actual on-disk structures
    may define different sizes, so use those unless creating a new GPT disk!
 */
@@ -246,6 +250,7 @@ typedef struct _GPTPartitionData {
 	int		lvm;
 	int		raid;
 	int		boot;
+	int		bios_grub;
 	int		hp_service;
         int             hidden;
         int             msftres;
@@ -281,7 +286,7 @@ pth_new (const PedDevice* dev)
         GuidPartitionTableHeader_t* pth = ped_malloc (
                         sizeof (GuidPartitionTableHeader_t)
                         + sizeof (uint8_t));
-        
+
         pth->Reserved2 = ped_malloc ( pth_get_size_rsv2 (dev) );
 
         return pth;
@@ -292,25 +297,25 @@ static GuidPartitionTableHeader_t*
 pth_new_zeroed (const PedDevice* dev)
 {
         GuidPartitionTableHeader_t* pth = pth_new (dev);
-        
+
         memset (pth, 0, pth_get_size_static (dev));
-        memset (pth->Reserved2, 0, pth_get_size_rsv2 (dev));        
-        
+        memset (pth->Reserved2, 0, pth_get_size_rsv2 (dev));
+
         return (pth);
 }
-                        
+
 
 static GuidPartitionTableHeader_t*
 pth_new_from_raw (const PedDevice* dev, const uint8_t* pth_raw)
 {
-        GuidPartitionTableHeader_t* pth = pth_new (dev); 
+        GuidPartitionTableHeader_t* pth = pth_new (dev);
 
         PED_ASSERT (pth_raw != NULL, return 0);
-        
+
         memcpy (pth, pth_raw, pth_get_size_static (dev));
         memcpy (pth->Reserved2, pth_raw + pth_get_size_static (dev),
                         pth_get_size_rsv2 (dev));
-        
+
         return pth;
 }
 
@@ -320,8 +325,8 @@ pth_free (GuidPartitionTableHeader_t* pth)
 	PED_ASSERT (pth != NULL, return);
 	PED_ASSERT (pth->Reserved2 != NULL, return);
 
-	ped_free (pth->Reserved2);
-	ped_free (pth);
+	free (pth->Reserved2);
+	free (pth);
 }
 
 static uint8_t*
@@ -329,13 +334,13 @@ pth_get_raw (const PedDevice* dev, const GuidPartitionTableHeader_t* pth)
 {
         uint8_t* pth_raw = ped_malloc (pth_get_size (dev));
         int size_static = pth_get_size_static (dev);
-        
+
         PED_ASSERT (pth != NULL, return 0);
         PED_ASSERT (pth->Reserved2 != NULL, return 0);
-       
+
         memcpy (pth_raw, pth, size_static);
         memcpy (pth_raw + size_static, pth->Reserved2, pth_get_size_rsv2 (dev));
-        
+
         return pth_raw;
 }
 
@@ -346,7 +351,7 @@ pth_get_raw (const PedDevice* dev, const GuidPartitionTableHeader_t* pth)
  *
  * There are two different representations for Globally Unique Identifiers
  * (GUIDs or UUIDs).
- * 
+ *
  * The RFC specifies a UUID as a string of 16 bytes, essentially
  * a big-endian array of char.
  * Intel, in their EFI Specification, references the same RFC, but
@@ -386,14 +391,14 @@ pth_crc32(const PedDevice* dev, const GuidPartitionTableHeader_t* pth)
 {
         uint8_t* pth_raw = pth_get_raw (dev, pth);
         uint32_t crc32 = 0;
-       
+
         PED_ASSERT (dev != NULL, return 0);
         PED_ASSERT (pth != NULL, return 0);
-       
-        crc32 = efi_crc32 (pth_raw, pth_get_size_static (dev));
 
-        ped_free (pth_raw);
-      
+        crc32 = efi_crc32 (pth_raw, PED_LE32_TO_CPU (pth->HeaderSize));
+
+        free (pth_raw);
+
         return crc32;
 }
 
@@ -436,13 +441,13 @@ gpt_probe (const PedDevice * dev)
 		if (gpt->Signature == PED_CPU_TO_LE64(GPT_HEADER_SIGNATURE))
 			gpt_sig_found = 1;
 	}
-	
-        ped_free (pth_raw);
+
+        free (pth_raw);
 
         if (gpt)
 	        pth_free (gpt);
 
-        
+
 	if (!gpt_sig_found)
 		return 0;
 
@@ -482,7 +487,7 @@ gpt_clobber(PedDevice * dev)
 
 	memset(&pmbr, 0, sizeof(pmbr));
 	memset(zeroed_pth_raw, 0, pth_get_size (dev));
-	
+
         /*
          * TO DISCUSS: check whether checksum is correct?
          * If not, we might get a wrong AlternateLBA field and destroy
@@ -493,7 +498,7 @@ gpt_clobber(PedDevice * dev)
                 goto error_free;
 
         gpt = pth_new_from_raw (dev, pth_raw);
-        
+
 	if (!ped_device_write(dev, &pmbr, GPT_PMBR_LBA, GPT_PMBR_SECTORS))
                 goto error_free_with_gpt;
 	if (!ped_device_write(dev, &zeroed_pth_raw,
@@ -509,16 +514,16 @@ gpt_clobber(PedDevice * dev)
 				      GPT_HEADER_SECTORS))
 			return 0;
 	}
-        
+
 	pth_free (gpt);
 
         return 1;
-        
+
 error_free_with_gpt:
         pth_free (gpt);
 error_free:
-        ped_free (pth_raw);
-        ped_free (zeroed_pth_raw);
+        free (pth_raw);
+        free (zeroed_pth_raw);
 	return 0;
 }
 #endif /* !DISCOVER_ONLY */
@@ -548,7 +553,7 @@ gpt_alloc (const PedDevice * dev)
 	return disk;
 
 error_free_disk:
-	ped_free (disk);
+	free (disk);
 error:
 	return NULL;
 }
@@ -579,7 +584,7 @@ static void
 gpt_free(PedDisk * disk)
 {
 	ped_disk_delete_all (disk);
-	ped_free (disk->disk_specific);
+	free (disk->disk_specific);
 	_ped_disk_free (disk);
 }
 
@@ -590,8 +595,21 @@ _header_is_valid (const PedDevice* dev, GuidPartitionTableHeader_t* gpt)
 
 	if (PED_LE64_TO_CPU (gpt->Signature) != GPT_HEADER_SIGNATURE)
 		return 0;
-	if (PED_LE32_TO_CPU (gpt->HeaderSize)
-			> pth_get_size_static (dev))
+	/*
+	 * "While the GUID Partition Table Header's size may increase
+	 * in the future it cannot span more than one block on the
+	 * device."  EFI Specification, version 1.10, 11.2.2.1
+	 */
+	if (PED_LE32_TO_CPU (gpt->HeaderSize) < pth_get_size_static (dev)
+	    || PED_LE32_TO_CPU (gpt->HeaderSize) > dev->sector_size)
+		return 0;
+
+	/*
+	 * the SizeOfPartitionEntry must be a multiple of 8 and
+	 * no smaller than the size of the PartitionEntry structure.
+	 */
+	uint32_t sope = gpt->SizeOfPartitionEntry;
+	if (sope % 8 != 0 || sope < sizeof(GuidPartitionEntry_t) )
 		return 0;
 
 	origcrc = gpt->HeaderCRC32;
@@ -611,13 +629,13 @@ _read_header (const PedDevice* dev, GuidPartitionTableHeader_t** gpt,
 	PED_ASSERT (dev != NULL, return 0);
 
 	if (!ped_device_read (dev, pth_raw, where, GPT_HEADER_SECTORS)) {
-                ped_free (pth_raw); 
+		free (pth_raw);
 		return 0;
         }
- 
+
         *gpt = pth_new_from_raw (dev, pth_raw);
-        
-        ped_free (pth_raw);
+
+        free (pth_raw);
 
         if (_header_is_valid (dev, *gpt))
                 return 1;
@@ -627,9 +645,9 @@ _read_header (const PedDevice* dev, GuidPartitionTableHeader_t** gpt,
 }
 
 static int
-_parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt, 
+_parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt,
 	       int *update_needed)
-{ 
+{
 	GPTDiskData* gpt_disk_data = disk->disk_specific;
 	PedSector first_usable;
 	PedSector last_usable;
@@ -639,9 +657,7 @@ _parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt,
 	PED_ASSERT (_header_is_valid (disk->dev, gpt), return 0);
 
 #ifndef DISCOVER_ONLY
-	if (PED_LE32_TO_CPU (gpt->Revision) > GPT_HEADER_REVISION_V1_02
-	    || PED_LE32_TO_CPU (gpt->HeaderSize) != pth_get_size_static (
-                                                        disk->dev)) {
+	if (PED_LE32_TO_CPU (gpt->Revision) > GPT_HEADER_REVISION_V1_02) {
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_IGNORE_CANCEL,
@@ -661,20 +677,20 @@ _parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt,
 /*
    Need to check whether the volume has grown, the LastUsableLBA is
    normally set to disk->dev->length - 2 - ptes_size (at least for parted
-   created volumes), where ptes_size is the number of entries * 
+   created volumes), where ptes_size is the number of entries *
    size of each entry / sector size or 16k / sector size, whatever the greater.
    If the volume has grown, offer the user the chance to use the new
-   space or continue with the current usable area.  Only ask once per 
+   space or continue with the current usable area.  Only ask once per
    parted invocation.
 */
-   
-	last_usable_if_grown 
-		= PED_CPU_TO_LE64 (disk->dev->length - 2 - 
-		((PedSector)(PED_LE32_TO_CPU(gpt->NumberOfPartitionEntries)) * 
-		(PedSector)(PED_LE32_TO_CPU(gpt->SizeOfPartitionEntry)) / 
+
+	last_usable_if_grown
+		= (disk->dev->length - 2 -
+		((PedSector)(PED_LE32_TO_CPU(gpt->NumberOfPartitionEntries)) *
+		(PedSector)(PED_LE32_TO_CPU(gpt->SizeOfPartitionEntry)) /
 		disk->dev->sector_size));
 
-	last_usable_min_default = disk->dev->length - 2 - 
+	last_usable_min_default = disk->dev->length - 2 -
 		GPT_DEFAULT_PARTITION_ENTRY_ARRAY_SIZE / disk->dev->sector_size;
 
 	if ( last_usable_if_grown > last_usable_min_default ) {
@@ -698,7 +714,7 @@ _parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt,
 			_("Not all of the space available to %s appears "
 			"to be used, you can fix the GPT to use all of the "
 			"space (an extra %llu blocks) or continue with the "
-			"current setting? "), disk->dev->path, 
+			"current setting? "), disk->dev->path,
 			(uint64_t)(last_usable_if_grown - last_usable));
 
 
@@ -735,7 +751,7 @@ _parse_part_entry (PedDisk* disk, GuidPartitionEntry_t* pte)
 	GPTPartitionData* gpt_part_data;
 	unsigned int i;
 
-	part = ped_partition_new (disk, 0, NULL,
+	part = ped_partition_new (disk, PED_PARTITION_NORMAL, NULL,
 			PED_LE64_TO_CPU(pte->StartingLBA),
 			PED_LE64_TO_CPU(pte->EndingLBA));
 	if (!part)
@@ -748,16 +764,19 @@ _parse_part_entry (PedDisk* disk, GuidPartitionEntry_t* pte)
 		gpt_part_data->name[i] = (efi_char16_t) PED_LE16_TO_CPU(
 					   (uint16_t) pte->PartitionName[i]);
 	gpt_part_data->name[i] = 0;
-        
-        gpt_part_data->lvm = gpt_part_data->raid 
+
+        gpt_part_data->lvm = gpt_part_data->raid
                 = gpt_part_data->boot = gpt_part_data->hp_service
-                = gpt_part_data->hidden = gpt_part_data->msftres = 0;
+                = gpt_part_data->hidden = gpt_part_data->msftres
+                = gpt_part_data->bios_grub = 0;
 
         if (pte->Attributes.RequiredToFunction & 0x1)
                 gpt_part_data->hidden = 1;
-       
+
 	if (!guid_cmp (gpt_part_data->type, PARTITION_SYSTEM_GUID))
 		gpt_part_data->boot = 1;
+	else if (!guid_cmp (gpt_part_data->type, PARTITION_BIOS_GRUB_GUID))
+		gpt_part_data->bios_grub = 1;
 	else if (!guid_cmp (gpt_part_data->type, PARTITION_RAID_GUID))
 		gpt_part_data->raid = 1;
 	else if (!guid_cmp (gpt_part_data->type, PARTITION_LVM_GUID))
@@ -766,7 +785,7 @@ _parse_part_entry (PedDisk* disk, GuidPartitionEntry_t* pte)
 		gpt_part_data->hp_service = 1;
         else if (!guid_cmp (gpt_part_data->type, PARTITION_MSFT_RESERVED_GUID))
                 gpt_part_data->msftres = 1;
-        
+
 	return part;
 }
 
@@ -797,8 +816,8 @@ gpt_read (PedDisk * disk)
 {
 	GPTDiskData *gpt_disk_data = disk->disk_specific;
 	GuidPartitionTableHeader_t* gpt;
-	GuidPartitionEntry_t* ptes;
-	int ptes_size;
+	void* ptes;
+	size_t ptes_sectors;
 	int i;
 #ifndef DISCOVER_ONLY
 	int write_back = 0;
@@ -806,7 +825,7 @@ gpt_read (PedDisk * disk)
 
 	ped_disk_delete_all (disk);
 
-        /* 
+        /*
          * motivation: let the user decide about the pmbr... during
          * ped_disk_probe(), they probably didn't get a choice...
          */
@@ -814,28 +833,42 @@ gpt_read (PedDisk * disk)
 		goto error;
 
 	if (_read_header (disk->dev, &gpt, 1)) {
-		PED_ASSERT ((PedSector) PED_LE64_TO_CPU (gpt->AlternateLBA)
-				<= disk->dev->length - 1, goto error_free_gpt);
+		/* There used to be a GPT partition table here, with an
+		   alternate LBA that extended beyond the current
+		   end-of-device.  Treat it as a non-match.   */
+		if ((PedSector) PED_LE64_TO_CPU (gpt->AlternateLBA)
+                    > disk->dev->length - 1)
+			goto error_free_gpt;
+
 		if ((PedSector) PED_LE64_TO_CPU (gpt->AlternateLBA)
 				< disk->dev->length - 1) {
-			char* zeros = ped_malloc (pth_get_size (disk->dev));
 
 #ifndef DISCOVER_ONLY
-			if (ped_exception_throw (
+			switch (ped_exception_throw (
 				PED_EXCEPTION_ERROR,
-				PED_EXCEPTION_FIX | PED_EXCEPTION_CANCEL,
+				PED_EXCEPTION_FIX | PED_EXCEPTION_CANCEL | PED_EXCEPTION_IGNORE,
 		_("The backup GPT table is not at the end of the disk, as it "
 		  "should be.  This might mean that another operating system "
 		  "believes the disk is smaller.  Fix, by moving the backup "
-		  "to the end (and removing the old backup)?"))
-					== PED_EXCEPTION_CANCEL)
-				goto error_free_gpt;
+		  "to the end (and removing the old backup)?"))) {
+				case PED_EXCEPTION_CANCEL:
+					goto error_free_gpt;
+				case PED_EXCEPTION_FIX:
+					{
+					char *zeros =
+					  ped_malloc (pth_get_size (disk->dev));
+					write_back = 1;
+					memset (zeros, 0, disk->dev->sector_size);
+					ped_device_write (disk->dev, zeros,
+							  PED_LE64_TO_CPU (gpt->AlternateLBA),
+							  1);
+					free (zeros);
+					}
+					break;
+				default:
+					break;
+			}
 
-			write_back = 1;
-			memset (zeros, 0, disk->dev->sector_size);
-			ped_device_write (disk->dev, zeros,
-					  PED_LE64_TO_CPU (gpt->AlternateLBA),
-					  1);
 #endif /* !DISCOVER_ONLY */
 		}
 	} else { /* primary GPT *not* ok */
@@ -878,22 +911,30 @@ gpt_read (PedDisk * disk)
 	if (!_parse_header (disk, gpt, &write_back))
 		goto error_free_gpt;
 
+	ptes_sectors = ped_div_round_up (gpt->SizeOfPartitionEntry
+					 * gpt_disk_data->entry_count,
+					 disk->dev->sector_size);
+
+	if (xalloc_oversized (ptes_sectors, disk->dev->sector_size))
+		goto error_free_gpt;
+	ptes = ped_malloc (ptes_sectors * disk->dev->sector_size);
 
-	ptes_size = sizeof (GuidPartitionEntry_t) * gpt_disk_data->entry_count;
-	ptes = (GuidPartitionEntry_t*) ped_malloc (ptes_size);
 	if (!ped_device_read (disk->dev, ptes,
 			      PED_LE64_TO_CPU(gpt->PartitionEntryLBA),
-			      ptes_size / disk->dev->sector_size))
+			      ptes_sectors))
 		goto error_free_ptes;
 
 	for (i = 0; i < gpt_disk_data->entry_count; i++) {
+		GuidPartitionEntry_t* pte
+		  = (GuidPartitionEntry_t*) ((char *)ptes + i
+					     * gpt->SizeOfPartitionEntry);
 		PedPartition* part;
 		PedConstraint* constraint_exact;
 
-		if (!guid_cmp (ptes[i].PartitionTypeGuid, UNUSED_ENTRY_GUID))
+		if (!guid_cmp (pte->PartitionTypeGuid, UNUSED_ENTRY_GUID))
 			continue;
 
-		part = _parse_part_entry (disk, &ptes[i]);
+		part = _parse_part_entry (disk, pte);
 		if (!part)
 			goto error_delete_all;
 
@@ -907,19 +948,20 @@ gpt_read (PedDisk * disk)
 		}
 		ped_constraint_destroy (constraint_exact);
 	}
-	ped_free (ptes);
+	free (ptes);
 
 #ifndef DISCOVER_ONLY
 	if (write_back)
 		ped_disk_commit_to_dev (disk);
 #endif
 
+	pth_free (gpt);
 	return 1;
 
 error_delete_all:
 	ped_disk_delete_all (disk);
 error_free_ptes:
-	ped_free (ptes);
+	free (ptes);
 error_free_gpt:
         pth_free (gpt);
 error:
@@ -927,13 +969,23 @@ error:
 }
 
 #ifndef DISCOVER_ONLY
-/* Writes the protective MBR (to keep DOS happy) */
+/* Write the protective MBR (to keep DOS happy) */
 static int
 _write_pmbr (PedDevice * dev)
 {
 	LegacyMBR_t pmbr;
 
-	memset(&pmbr, 0, sizeof(pmbr));
+	/* The UEFI spec is not clear about what to do with the following
+	   elements of the Protective MBR (pmbr): BootCode (0-440B),
+	   UniqueMBRSignature (440B-444B) and Unknown (444B-446B).
+	   With this in mind, we try not to modify these elements.  */
+	if (ped_device_read (dev, &pmbr, 0, GPT_PMBR_SECTORS) < GPT_PMBR_SECTORS)
+		memset (&pmbr, 0, sizeof(pmbr));
+
+	/* Zero out all the legacy partitions.
+	   There are 4 PartitionRecords.  */
+	memset (pmbr.PartitionRecord, 0, sizeof pmbr.PartitionRecord);
+
 	pmbr.Signature = PED_CPU_TO_LE16(MSDOS_MBR_SIGNATURE);
 	pmbr.PartitionRecord[0].OSType      = EFI_PMBR_OSTYPE_EFI;
 	pmbr.PartitionRecord[0].StartSector = 1;
@@ -941,7 +993,7 @@ _write_pmbr (PedDevice * dev)
 	pmbr.PartitionRecord[0].EndSector   = 0xFF;
 	pmbr.PartitionRecord[0].EndTrack    = 0xFF;
 	pmbr.PartitionRecord[0].StartingLBA = PED_CPU_TO_LE32(1);
-	if ((dev->length - 1ULL) > 0xFFFFFFFFULL) 
+	if ((dev->length - 1ULL) > 0xFFFFFFFFULL)
 		pmbr.PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(0xFFFFFFFF);
 	else
 		pmbr.PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(dev->length - 1UL);
@@ -957,9 +1009,9 @@ _generate_header (const PedDisk* disk, int alternate, uint32_t ptes_crc,
         GuidPartitionTableHeader_t* gpt;
 
         *gpt_p = pth_new_zeroed (disk->dev);
-        
+
         gpt = *gpt_p;
-        
+
 	gpt->Signature = PED_CPU_TO_LE64 (GPT_HEADER_SIGNATURE);
 	gpt->Revision = PED_CPU_TO_LE32 (GPT_HEADER_REVISION_V1_00);
 
@@ -1009,7 +1061,7 @@ _partition_generate_part_entry (PedPartition* part, GuidPartitionEntry_t* pte)
 
         if (gpt_part_data->hidden)
                 pte->Attributes.RequiredToFunction = 1;
-        
+
 	for (i = 0; i < 72 / sizeof(efi_char16_t); i++)
 		pte->PartitionName[i]
 			= (efi_char16_t) PED_CPU_TO_LE16(
@@ -1022,7 +1074,7 @@ gpt_write(const PedDisk * disk)
 	GPTDiskData* gpt_disk_data;
 	GuidPartitionEntry_t* ptes;
 	uint32_t ptes_crc;
-        uint8_t* pth_raw = ped_malloc (pth_get_size (disk->dev));
+	uint8_t* pth_raw;
 	GuidPartitionTableHeader_t* gpt;
 	PedPartition* part;
 	int ptes_size;
@@ -1053,27 +1105,35 @@ gpt_write(const PedDisk * disk)
 
 	/* Write PTH and PTEs */
 	_generate_header (disk, 0, ptes_crc, &gpt);
-        pth_raw = pth_get_raw (disk->dev, gpt);
-	if (!ped_device_write (disk->dev, pth_raw, 1, 1))
+	pth_raw = pth_get_raw (disk->dev, gpt);
+	pth_free (gpt);
+	bool write_ok = ped_device_write (disk->dev, pth_raw, 1, 1);
+	free (pth_raw);
+	if (!write_ok)
 		goto error_free_ptes;
-	if (!ped_device_write (disk->dev, ptes, 2, ptes_size / disk->dev->sector_size))
+	if (!ped_device_write (disk->dev, ptes, 2,
+			       ptes_size / disk->dev->sector_size))
 		goto error_free_ptes;
 
 	/* Write Alternate PTH & PTEs */
 	_generate_header (disk, 1, ptes_crc, &gpt);
         pth_raw = pth_get_raw (disk->dev, gpt);
-	if (!ped_device_write (disk->dev, pth_raw, disk->dev->length - 1, 1))
+        pth_free (gpt);
+	write_ok = ped_device_write (disk->dev, pth_raw,
+				     disk->dev->length - 1, 1);
+	free (pth_raw);
+	if (!write_ok)
 		goto error_free_ptes;
 	if (!ped_device_write (disk->dev, ptes,
 			       disk->dev->length - 1 - ptes_size / disk->dev->sector_size,
 			       ptes_size / disk->dev->sector_size))
 		goto error_free_ptes;
 
-	ped_free (ptes);
+	free (ptes);
 	return ped_device_sync (disk->dev);
 
 error_free_ptes:
-	ped_free (ptes);
+	free (ptes);
 error:
 	return 0;
 }
@@ -1128,12 +1188,13 @@ gpt_partition_new (const PedDisk* disk,
 	gpt_part_data->lvm = 0;
 	gpt_part_data->raid = 0;
 	gpt_part_data->boot = 0;
+	gpt_part_data->bios_grub = 0;
 	gpt_part_data->hp_service = 0;
         gpt_part_data->hidden = 0;
         gpt_part_data->msftres = 0;
 	uuid_generate ((unsigned char*) &gpt_part_data->uuid);
 	swap_uuid_and_efi_guid((unsigned char*)(&gpt_part_data->uuid));
-	strcpy (gpt_part_data->name, "");
+	memset (gpt_part_data->name, 0, sizeof gpt_part_data->name);
 	return part;
 
 error_free_part:
@@ -1179,7 +1240,7 @@ gpt_partition_destroy (PedPartition *part)
 {
 	if (part->type == 0) {
 		PED_ASSERT (part->disk_specific != NULL, return);
-		ped_free (part->disk_specific);
+		free (part->disk_specific);
 	}
 
 	_ped_partition_free (part);
@@ -1206,6 +1267,10 @@ gpt_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		gpt_part_data->type = PARTITION_SYSTEM_GUID;
 		return 1;
 	}
+	if (gpt_part_data->bios_grub) {
+		gpt_part_data->type = PARTITION_BIOS_GRUB_GUID;
+		return 1;
+	}
 	if (gpt_part_data->hp_service) {
 		gpt_part_data->type = PARTITION_HPSERVICE_GUID;
 		return 1;
@@ -1214,7 +1279,7 @@ gpt_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
                 gpt_part_data->type = PARTITION_MSFT_RESERVED_GUID;
                 return 1;
         }
-        
+
 	if (fs_type) {
 		if (strncmp (fs_type->name, "fat", 3) == 0
 		    || strcmp (fs_type->name, "ntfs") == 0) {
@@ -1302,8 +1367,18 @@ gpt_partition_set_flag(PedPartition *part,
 	case PED_PARTITION_BOOT:
 		gpt_part_data->boot = state;
 		if (state)
-                        gpt_part_data->raid 
+                        gpt_part_data->raid
                                 = gpt_part_data->lvm
+                                = gpt_part_data->bios_grub
+                                = gpt_part_data->hp_service
+                                = gpt_part_data->msftres = 0;
+		return gpt_partition_set_system (part, part->fs_type);
+	case PED_PARTITION_BIOS_GRUB:
+		gpt_part_data->bios_grub = state;
+		if (state)
+                        gpt_part_data->raid
+                                = gpt_part_data->lvm
+                                = gpt_part_data->boot
                                 = gpt_part_data->hp_service
                                 = gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
@@ -1312,6 +1387,7 @@ gpt_partition_set_flag(PedPartition *part,
 		if (state)
                         gpt_part_data->boot
                                 = gpt_part_data->lvm
+                                = gpt_part_data->bios_grub
                                 = gpt_part_data->hp_service
                                 = gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
@@ -1320,6 +1396,7 @@ gpt_partition_set_flag(PedPartition *part,
 		if (state)
                         gpt_part_data->boot
                                 = gpt_part_data->raid
+                                = gpt_part_data->bios_grub
                                 = gpt_part_data->hp_service
                                 = gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
@@ -1329,6 +1406,7 @@ gpt_partition_set_flag(PedPartition *part,
                         gpt_part_data->boot
                                 = gpt_part_data->raid
                                 = gpt_part_data->lvm
+                                = gpt_part_data->bios_grub
                                 = gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
         case PED_PARTITION_MSFT_RESERVED:
@@ -1337,6 +1415,7 @@ gpt_partition_set_flag(PedPartition *part,
                         gpt_part_data->boot
                                 = gpt_part_data->raid
                                 = gpt_part_data->lvm
+                                = gpt_part_data->bios_grub
                                 = gpt_part_data->hp_service = 0;
                 return gpt_partition_set_system (part, part->fs_type);
         case PED_PARTITION_HIDDEN:
@@ -1365,6 +1444,8 @@ gpt_partition_get_flag(const PedPartition *part, PedPartitionFlag flag)
 		return gpt_part_data->lvm;
 	case PED_PARTITION_BOOT:
 		return gpt_part_data->boot;
+	case PED_PARTITION_BIOS_GRUB:
+		return gpt_part_data->bios_grub;
 	case PED_PARTITION_HPSERVICE:
 		return gpt_part_data->hp_service;
         case PED_PARTITION_MSFT_RESERVED:
@@ -1388,9 +1469,10 @@ gpt_partition_is_flag_available(const PedPartition * part,
 	case PED_PARTITION_RAID:
 	case PED_PARTITION_LVM:
 	case PED_PARTITION_BOOT:
+	case PED_PARTITION_BIOS_GRUB:
 	case PED_PARTITION_HPSERVICE:
         case PED_PARTITION_MSFT_RESERVED:
-        case PED_PARTITION_HIDDEN:        
+        case PED_PARTITION_HIDDEN:
 		return 1;
 	case PED_PARTITION_SWAP:
 	case PED_PARTITION_ROOT:
@@ -1424,6 +1506,52 @@ gpt_get_max_primary_partition_count (const PedDisk *disk)
 	return gpt_disk_data->entry_count;
 }
 
+/*
+ * From (http://developer.apple.com/technotes/tn2006/tn2166.html Chapter 5).
+ * According to the specs the first LBA (LBA0) is not relevant (it exists
+ * to maintain compatibility).  on the second LBA(LBA1) gpt places the
+ * header.  The header is as big as the block size.  After the header we
+ * find the Entry array.  Each element of said array, describes each
+ * partition.  One can have as much elements as can fit between the end of
+ * the second LBA (where the header ends) and the FirstUsableLBA.
+ * FirstUsableLBA is the first logical block that is used for contents
+ * and is defined in header.
+ *
+ * /---------------------------------------------------\
+ * | BLOCK0 | HEADER | Entry Array | First Usable LBA  |
+ * |        | BLOCK1 |             |                   |
+ * \---------------------------------------------------/
+ *                  /              \
+ *     /----------/                  \----------\
+ *     /-----------------------------------------\
+ *     |  E1  |  E2  |  E3  |...............| EN |
+ *     \-----------------------------------------/
+ *
+ * The number of possible partitions or supported partitions is:
+ * SP = FirstUsableLBA*Blocksize - 2*Blocksize / SizeOfPartitionEntry
+ * SP = Blocksize(FirstusableLBA - 2) / SizeOfPartitoinEntry
+ */
+static bool
+gpt_get_max_supported_partition_count (const PedDisk *disk, int *max_n)
+{
+	GuidPartitionTableHeader_t *pth = NULL;
+	uint8_t *pth_raw = ped_malloc (pth_get_size (disk->dev));
+
+	if (ped_device_read (disk->dev, pth_raw, 1, GPT_HEADER_SECTORS)
+	    || ped_device_read (disk->dev, pth_raw,
+				disk->dev->length, GPT_HEADER_SECTORS))
+		pth = pth_new_from_raw (disk->dev, pth_raw);
+	free (pth_raw);
+
+	if (pth == NULL)
+		return false;
+
+	*max_n = (disk->dev->sector_size * (pth->FirstUsableLBA - 2)
+		  / pth->SizeOfPartitionEntry);
+	pth_free (pth);
+	return true;
+}
+
 static PedConstraint*
 _non_metadata_constraint (const PedDisk* disk)
 {
@@ -1478,7 +1606,8 @@ static PedDiskOps gpt_disk_ops = {
 	partition_align:		gpt_partition_align,
 	partition_enumerate:		gpt_partition_enumerate,
 	alloc_metadata:			gpt_alloc_metadata,
-	get_max_primary_partition_count: gpt_get_max_primary_partition_count
+	get_max_primary_partition_count: gpt_get_max_primary_partition_count,
+	get_max_supported_partition_count: gpt_get_max_supported_partition_count
 };
 
 static PedDiskType gpt_disk_type = {
