@@ -1,11 +1,11 @@
 /* -*- Mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2001, 2005 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2001, 2005, 2007-2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -14,19 +14,17 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
     Contributor:  Ben Collins <bcollins@debian.org>
 */
 
-#include "config.h"
-
-#include <string.h>
+#include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
 #include <parted/endian.h>
+#include <stdbool.h>
 
 #if ENABLE_NLS
 #  include <libintl.h>
@@ -35,6 +33,8 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
+#include "misc.h"
+
 /* Most of this came from util-linux's sun support, which was mostly done
    by Jakub Jelinek.  */
 
@@ -45,19 +45,25 @@
 #define WHOLE_DISK_PART		2	/* as in 0, 1, 2 (3rd partition) */
 #define LINUX_SWAP_ID		0x82
 
-typedef struct {
+typedef struct _SunRawPartition     SunRawPartition;
+typedef struct _SunPartitionInfo    SunPartitionInfo;
+typedef struct _SunRawLabel         SunRawLabel;
+typedef struct _SunPartitionData    SunPartitionData;
+typedef struct _SunDiskData         SunDiskData;
+
+struct __attribute__ ((packed)) _SunRawPartition {
 	u_int32_t	start_cylinder; /* where the part starts... */
 	u_int32_t	num_sectors;	/* ...and it's length */
-} __attribute__ ((packed)) SunRawPartition;
+};
 
-typedef struct {
+struct __attribute__ ((packed)) _SunPartitionInfo {
 	u_int8_t	spare1;
 	u_int8_t	id;		/* Partition type */
 	u_int8_t	spare2;
 	u_int8_t	flags;		/* Partition flags */
-} __attribute__ ((packed)) SunPartitionInfo;
+};
 
-typedef struct {
+struct __attribute__ ((packed)) _SunRawLabel {
 	char 		info[128];	/* Informative text string */
 	u_int8_t	spare0[14];
 	SunPartitionInfo infos[SUN_DISK_MAXPARTITIONS];
@@ -75,19 +81,20 @@ typedef struct {
 	SunRawPartition partitions[SUN_DISK_MAXPARTITIONS];
 	u_int16_t	magic;		/* Magic number */
 	u_int16_t	csum;		/* Label xor'd checksum */
-}  __attribute__ ((packed)) SunRawLabel;
+};
 
-typedef struct {
+struct _SunPartitionData {
 	u_int8_t		type;
 	int			is_boot;
 	int			is_root;
 	int			is_lvm;
-} SunPartitionData;
+	int			is_raid;
+};
 
-typedef struct {
+struct _SunDiskData {
 	PedSector		length; /* This is based on cyl - alt-cyl */
 	SunRawLabel		raw_label;
-} SunDiskData;
+};
 
 static PedDiskType sun_disk_type;
 
@@ -119,9 +126,7 @@ sun_verify_checksum (SunRawLabel *label)
 static int
 sun_probe (const PedDevice *dev)
 {
-	PedDiskType*	disk_type;
 	SunRawLabel	label;
-	int		i;
 
 	PED_ASSERT (dev != NULL, return 0);
 
@@ -159,7 +164,7 @@ sun_clobber (PedDevice* dev)
 
 	if (!ped_device_read (dev, &label, 0, 1))
 		return 0;
-	
+
 	label.magic = 0;
 	return ped_device_write (dev, &label, 0, 1);
 }
@@ -229,7 +234,7 @@ sun_duplicate (const PedDisk* disk)
 	PedDisk*	new_disk;
 	SunDiskData*	new_sun_data;
 	SunDiskData*	old_sun_data = (SunDiskData*) disk->disk_specific;
-       
+
 	new_disk = ped_disk_new_fresh (disk->dev, &sun_disk_type);
 	if (!new_disk)
 		return NULL;
@@ -242,7 +247,7 @@ sun_duplicate (const PedDisk* disk)
 static void
 sun_free (PedDisk *disk)
 {
-	ped_free (disk->disk_specific);
+	free (disk->disk_specific);
 	_ped_disk_free (disk);
 }
 
@@ -303,11 +308,11 @@ sun_read (PedDisk* disk)
 	PedPartition* part;
 	PedSector end, start, block;
 	PedConstraint* constraint_exact;
-	
+
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (disk->dev != NULL, return 0);
 	PED_ASSERT (disk->disk_specific != NULL, return 0);
-	
+
 	disk_data = (SunDiskData*) disk->disk_specific;
 	label = &disk_data->raw_label;
 
@@ -334,7 +339,8 @@ sun_read (PedDisk* disk)
 		end = start
 		      + PED_BE32_TO_CPU(label->partitions[i].num_sectors) - 1;
 
-		part = ped_partition_new (disk, 0, NULL, start, end);
+		part = ped_partition_new (disk, PED_PARTITION_NORMAL, NULL,
+                                          start, end);
 		if (!part)
 			goto error;
 
@@ -343,10 +349,11 @@ sun_read (PedDisk* disk)
 		sun_data->is_boot = sun_data->type == 0x1;
 		sun_data->is_root = sun_data->type == 0x2;
 		sun_data->is_lvm = sun_data->type == 0x8e;
+		sun_data->is_raid = sun_data->type == 0xfd;
 
 		part->num = i + 1;
 		part->fs_type = ped_file_system_probe (&part->geom);
-	
+
 		constraint_exact = ped_constraint_exact (&part->geom);
 		if (!ped_disk_add_partition (disk, part, constraint_exact))
 			goto error;
@@ -361,7 +368,7 @@ sun_read (PedDisk* disk)
 
 #ifndef DISCOVER_ONLY
 static void
-_probe_and_use_old_info (PedDisk* disk)
+_probe_and_use_old_info (const PedDisk* disk)
 {
 	SunDiskData*		sun_specific;
 	SunRawLabel		old_label;
@@ -376,7 +383,7 @@ _probe_and_use_old_info (PedDisk* disk)
 }
 
 static int
-sun_write (PedDisk* disk)
+sun_write (const PedDisk* disk)
 {
 	SunRawLabel*		label;
 	SunPartitionData*	sun_data;
@@ -407,7 +414,7 @@ sun_write (PedDisk* disk)
 
 			label->infos[i].id = WHOLE_DISK_ID;
 			label->partitions[i].start_cylinder = 0;
-			label->partitions[i].num_sectors = 
+			label->partitions[i].num_sectors =
 				PED_CPU_TO_BE32(disk_data->length);
 			continue;
 		}
@@ -478,16 +485,16 @@ sun_partition_new (const PedDisk* disk, PedPartitionType part_type,
 		sun_data->is_boot = 0;
 		sun_data->is_root = 0;
 		sun_data->is_lvm = 0;
+		sun_data->is_raid = 0;
 	} else {
 		part->disk_specific = NULL;
 	}
 
 	return part;
 
-error_free_sun_data:
-	ped_free (sun_data);
+	free (sun_data);
 error_free_part:
-	ped_free (part);
+	free (part);
 error:
 	return NULL;
 }
@@ -512,6 +519,7 @@ sun_partition_duplicate (const PedPartition* part)
 	new_sun_data->is_boot = old_sun_data->is_boot;
 	new_sun_data->is_root = old_sun_data->is_root;
 	new_sun_data->is_lvm = old_sun_data->is_lvm;
+	new_sun_data->is_raid = old_sun_data->is_raid;
 	return new_part;
 }
 
@@ -521,8 +529,8 @@ sun_partition_destroy (PedPartition* part)
 	PED_ASSERT (part != NULL, return);
 
 	if (ped_partition_is_active (part))
-		ped_free (part->disk_specific);
-	ped_free (part);
+		free (part->disk_specific);
+	free (part);
 }
 
 static int
@@ -544,10 +552,14 @@ sun_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		sun_data->type = 0x8e;
 		return 1;
 	}
+	if (sun_data->is_raid) {
+		sun_data->type = 0xfd;
+		return 1;
+	}
 
 	sun_data->type = 0x83;
 	if (fs_type) {
-		if (!strcmp (fs_type->name, "linux-swap"))
+		if (is_linux_swap (fs_type->name))
 			sun_data->type = 0x82;
 		else if (!strcmp (fs_type->name, "ufs"))
 			sun_data->type = 0x6;
@@ -570,20 +582,38 @@ sun_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 	switch (flag) {
 		case PED_PARTITION_BOOT:
 			sun_data->is_boot = state;
-			if (state)
-				sun_data->is_root = sun_data->is_lvm = 0;
+			if (state) {
+				sun_data->is_lvm = 0;
+				sun_data->is_raid = 0;
+				sun_data->is_root = 0;
+			}
 			return ped_partition_set_system (part, part->fs_type);
 
 		case PED_PARTITION_ROOT:
 			sun_data->is_root = state;
-			if (state)
-				sun_data->is_boot = sun_data->is_lvm = 0;
+			if (state) {
+				sun_data->is_boot = 0;
+				sun_data->is_lvm = 0;
+				sun_data->is_raid = 0;
+			}
 			return ped_partition_set_system (part, part->fs_type);
 
 		case PED_PARTITION_LVM:
 			sun_data->is_lvm = state;
-			if (state)
-				sun_data->is_root = sun_data->is_boot = 0;
+			if (state) {
+				sun_data->is_boot = 0;
+				sun_data->is_raid = 0;
+				sun_data->is_root = 0;
+			}
+			return ped_partition_set_system (part, part->fs_type);
+
+		case PED_PARTITION_RAID:
+			sun_data->is_raid = state;
+			if (state) {
+				sun_data->is_boot = 0;
+				sun_data->is_lvm = 0;
+				sun_data->is_root = 0;
+			}
 			return ped_partition_set_system (part, part->fs_type);
 
 		default:
@@ -609,6 +639,8 @@ sun_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 			return sun_data->is_root;
 		case PED_PARTITION_LVM:
 			return sun_data->is_lvm;
+		case PED_PARTITION_RAID:
+			return sun_data->is_raid;
 
 		default:
 			return 0;
@@ -624,6 +656,7 @@ sun_partition_is_flag_available (const PedPartition* part,
 		case PED_PARTITION_BOOT:
 		case PED_PARTITION_ROOT:
 		case PED_PARTITION_LVM:
+		case PED_PARTITION_RAID:
 			return 1;
 
 		default:
@@ -631,6 +664,12 @@ sun_partition_is_flag_available (const PedPartition* part,
 	}
 }
 
+static bool
+sun_get_max_supported_partition_count (const PedDisk* disk, int *max_n)
+{
+	*max_n = SUN_DISK_MAXPARTITIONS;
+	return true;
+}
 
 static int
 sun_get_max_primary_partition_count (const PedDisk* disk)
@@ -711,7 +750,7 @@ sun_partition_enumerate (PedPartition* part)
 {
 	int i;
 	PedPartition* p;
-	
+
 	/* never change the partition numbers */
 	if (part->num != -1)
 		return 1;
@@ -730,7 +769,7 @@ sun_partition_enumerate (PedPartition* part)
 	/* Ok, now allocate the Whole disk if it isn't already */
 	p = ped_disk_get_partition (part->disk, WHOLE_DISK_PART + 1);
 	if (!p) {
-		int i = ped_exception_throw (
+		int j = ped_exception_throw (
 				PED_EXCEPTION_WARNING,
 				PED_EXCEPTION_IGNORE_CANCEL,
 				_("The Whole Disk partition is the only "
@@ -739,7 +778,7 @@ sun_partition_enumerate (PedPartition* part)
 				  "a real one.  Solaris may not be able to "
 				  "boot without it, and SILO (the sparc boot "
 				  "loader) appreciates it as well."));
-		if (i == PED_EXCEPTION_IGNORE) {
+		if (j == PED_EXCEPTION_IGNORE) {
 			/* bad bad bad...you will suffer your own fate */
 			part->num = WHOLE_DISK_PART + 1;
 			return 1;
@@ -826,6 +865,8 @@ static PedDiskOps sun_disk_ops = {
 	alloc_metadata:		sun_alloc_metadata,
 	get_max_primary_partition_count:
 				sun_get_max_primary_partition_count,
+	get_max_supported_partition_count:
+				sun_get_max_supported_partition_count,
 
 	partition_set_name:		NULL,
 	partition_get_name:		NULL,
@@ -842,12 +883,11 @@ void
 ped_disk_sun_init ()
 {
 	PED_ASSERT (sizeof (SunRawLabel) == 512, return);
-	ped_register_disk_type (&sun_disk_type);
+	ped_disk_type_register (&sun_disk_type);
 }
 
 void
 ped_disk_sun_done ()
 {
-	ped_unregister_disk_type (&sun_disk_type);
+	ped_disk_type_unregister (&sun_disk_type);
 }
-
