@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2007, 2008 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,13 +22,7 @@
 #include <parted/parted.h>
 #include <parted/debug.h>
 
-#ifdef linux
-#  include <parted/linux.h>
-#elif defined(__BEOS__)
-#  include <parted/beos.h>
-#else
-#  include <parted/gnu.h>
-#endif
+#include "architecture.h"
 
 #if ENABLE_NLS
 #  include <locale.h>
@@ -38,10 +32,8 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
-const PedArchitecture* ped_architecture;
-
 /* ped_malloc() debugging.  Stick the address and size of memory blocks that
- * weren't ped_free()d in here, and an exception will be thrown when it is
+ * weren't free()d in here, and an exception will be thrown when it is
  * allocated.  That way, you can find out what, exactly, the allocated thing
  * is, and where it is created.
  */
@@ -72,15 +64,6 @@ static pointer_size_type dodgy_malloc_list[] = {
 static int	dodgy_memory_active[100];
 #endif /* DEBUG */
 
-int
-ped_set_architecture (const PedArchitecture* arch)
-{
-	PED_ASSERT (ped_device_get_next (NULL) == NULL, return 0);
-
-	ped_architecture = arch;
-	return 1;
-}
-
 extern void ped_disk_aix_init ();
 extern void ped_disk_bsd_init ();
 extern void ped_disk_dvh_init ();
@@ -189,16 +172,7 @@ _init()
 #ifdef ENABLE_FS
 	init_file_system_types ();
 #endif
-
-	/* FIXME: a better way of doing this? */
-#ifdef linux
-	ped_set_architecture (&ped_linux_arch);
-#elif defined(__BEOS__)
-	ped_set_architecture (&ped_beos_arch);
-#else
-	ped_set_architecture (&ped_gnu_arch);
-#endif
-
+	ped_set_architecture ();
 #ifdef DEBUG
 	memset (dodgy_memory_active, 0, sizeof (dodgy_memory_active));
 #endif
@@ -252,42 +226,6 @@ ped_get_version ()
 	return VERSION;
 }
 
-#ifdef DEBUG
-static void
-_check_dodgy_pointer (const void* ptr, size_t size, int is_malloc)
-{
-	int		i;
-
-	for (i=0; dodgy_malloc_list[i].pointer; i++) {
-		if (dodgy_malloc_list[i].pointer != ptr)
- 			continue;
-		if (is_malloc && dodgy_malloc_list[i].size != size)
-			continue;
-		if (!is_malloc && !dodgy_memory_active[i])
-			continue;
-
-
-		if (is_malloc) {
-			ped_exception_throw (
-				PED_EXCEPTION_INFORMATION,
-				PED_EXCEPTION_OK,
-				"Dodgy malloc(%x) == %p occurred (active==%d)",
-				size, ptr, dodgy_memory_active[i]);
-			dodgy_memory_active[i]++;
-		} else {
-			ped_exception_throw (
-				PED_EXCEPTION_INFORMATION,
-				PED_EXCEPTION_OK,
-				"Dodgy free(%p) occurred (active==%d)",
-				ptr, dodgy_memory_active[i]);
-			dodgy_memory_active[i]--;
-		}
-
-		return;
-	}
-}
-#endif /* DEBUG */
-
 void*
 ped_malloc (size_t size)
 {
@@ -300,11 +238,6 @@ ped_malloc (size_t size)
 		return NULL;
 	}
 
-#ifdef DEBUG
-	memset (mem, 0xff, size);
-	_check_dodgy_pointer (mem, size, 1);
-#endif
-
 	return mem;
 }
 
@@ -332,14 +265,3 @@ void* ped_calloc (size_t size)
 
         return buf;
 }
-
-
-void
-ped_free (void* ptr)
-{
-#ifdef DEBUG
-	_check_dodgy_pointer (ptr, 0, 0);
-#endif
-
-	free (ptr);
-}
