@@ -20,26 +20,26 @@
 
 /**
  * \addtogroup PedUnit
- * 
- * \brief The PedUnit module provides a standard mechanism for describing
- * and parsing locations within devices in human-friendly plain text. 
  *
- * Internally, libparted uses PedSector (which is typedef'ed to be long long 
- * in <parted/device.h>) to describe device locations such as the start and 
- * end of partitions.  However, sector numbers are often long and unintuitive.  
- * For example, my extended partition starts at sector 208845.  PedUnit allows 
- * this location to be represented in more intutitive ways, including "106Mb", 
- * "0Gb" and "0%", as well as "208845s".  PedUnit aims to provide facilities 
+ * \brief The PedUnit module provides a standard mechanism for describing
+ * and parsing locations within devices in human-friendly plain text.
+ *
+ * Internally, libparted uses PedSector (which is typedef'ed to be long long
+ * in <parted/device.h>) to describe device locations such as the start and
+ * end of partitions.  However, sector numbers are often long and unintuitive.
+ * For example, my extended partition starts at sector 208845.  PedUnit allows
+ * this location to be represented in more intutitive ways, including "106Mb",
+ * "0Gb" and "0%", as well as "208845s".  PedUnit aims to provide facilities
  * to provide a consistent system for describing device locations all
  * throughout libparted.
- * 
+ *
  * PedUnit provides two basic services: converting a PedSector into a text
  * representation, and parsing a text representation into a PedSector.
  * PedUnit currently supports these units:
- * 
+ *
  * 	sectors, bytes, kilobytes, megabytes, gigabytes, terabytes, compact,
  * 	cylinder and percent.
- * 	
+ *
  * PedUnit has a global variable that contains the default unit for all
  * conversions.
  *
@@ -147,7 +147,7 @@ ped_unit_get_size (const PedDevice* dev, PedUnit unit)
 
 /**
  * Get a textual (non-internationalized) representation of a \p unit.
- * 
+ *
  * For example, the textual representation of PED_UNIT_SECTOR is "s".
  */
 const char*
@@ -186,9 +186,9 @@ ped_strdup (const char *str)
 /**
  * \brief Get a string that describes the location of the \p byte on
  * device \p dev.
- * 
+ *
  * The string is described with the desired \p unit.
- * The returned string must be freed with ped_free().
+ * The returned string must be freed with free().
  */
 char*
 ped_unit_format_custom_byte (const PedDevice* dev, PedSector byte, PedUnit unit)
@@ -199,7 +199,7 @@ ped_unit_format_custom_byte (const PedDevice* dev, PedSector byte, PedUnit unit)
 	int p;
 
 	PED_ASSERT (dev != NULL, return NULL);
-	
+
 	/* CHS has a special comma-separated format. */
 	if (unit == PED_UNIT_CHS) {
 		const PedCHSGeometry *chs = &dev->bios_geom;
@@ -219,7 +219,7 @@ ped_unit_format_custom_byte (const PedDevice* dev, PedSector byte, PedUnit unit)
 			  ped_unit_get_name (unit));
 		return ped_strdup (buf);
 	}
-	
+
         if (unit == PED_UNIT_COMPACT) {
                 if (byte >= 10LL * PED_TERABYTE_SIZE)
                         unit = PED_UNIT_TERABYTE;
@@ -258,10 +258,10 @@ ped_unit_format_custom_byte (const PedDevice* dev, PedSector byte, PedUnit unit)
 /**
  * \brief Get a string that describes the location of the \p byte on
  * device \p dev.
- * 
+ *
  * The string is described with the default unit, which is set
  * by ped_unit_set_default().
- * The returned string must be freed with ped_free().
+ * The returned string must be freed with free().
  */
 char*
 ped_unit_format_byte (const PedDevice* dev, PedSector byte)
@@ -272,9 +272,9 @@ ped_unit_format_byte (const PedDevice* dev, PedSector byte)
 
 /**
  * \brief Get a string that describes the location \p sector on device \p dev.
- * 
+ *
  * The string is described with the desired \p unit.
- * The returned string must be freed with ped_free().
+ * The returned string must be freed with free().
  */
 char*
 ped_unit_format_custom (const PedDevice* dev, PedSector sector, PedUnit unit)
@@ -285,10 +285,10 @@ ped_unit_format_custom (const PedDevice* dev, PedSector sector, PedUnit unit)
 
 /**
  * \brief Get a string that describes the location \p sector on device \p dev.
- * 
+ *
  * The string is described with the default unit, which is set
  * by ped_unit_set_default().
- * The returned string must be freed with ped_free().
+ * The returned string must be freed with free().
  */
 char*
 ped_unit_format (const PedDevice* dev, PedSector sector)
@@ -299,13 +299,13 @@ ped_unit_format (const PedDevice* dev, PedSector sector)
 }
 
 /**
- * If \p str contains a valid description of a location on \p dev, 
- * then \p *sector is modified to describe the location and a geometry 
- * is created in \p *range describing a 2 units large area centered on 
- * \p *sector.  If the \p range as described here would be partially outside 
- * the device \p dev, the geometry returned is the intersection between the 
- * former and the whole	device geometry.  If no units are specified, then the 
- * default unit is assumed.  
+ * If \p str contains a valid description of a location on \p dev,
+ * then \p *sector is modified to describe the location and a geometry
+ * is created in \p *range describing a 2 units large area centered on
+ * \p *sector.  If the \p range as described here would be partially outside
+ * the device \p dev, the geometry returned is the intersection between the
+ * former and the whole	device geometry.  If no units are specified, then the
+ * default unit is assumed.
  *
  * \return \c 1 if \p str is a valid location description, \c 0 otherwise
  */
@@ -416,11 +416,11 @@ parse_chs (const char* str, const PedDevice* dev, PedSector* sector,
 	}
 	if (range)
 		*range = ped_geometry_new (dev, *sector, 1);
-	ped_free (copy);
+	free (copy);
 	return !range || *range != NULL;
 
 error_free_copy:
-	ped_free (copy);
+	free (copy);
 	*sector = 0;
 	if (range)
 		*range = NULL;
@@ -464,9 +464,9 @@ parse_unit_suffix (const char* suffix, PedUnit suggested_unit)
 			case 'b': return PED_UNIT_BYTE;
 			case 'k': return PED_UNIT_KILOBYTE;
 			case 'm': return PED_UNIT_MEGABYTE;
-			case 'g': return PED_UNIT_GIGABYTE; 
+			case 'g': return PED_UNIT_GIGABYTE;
 			case 't': return PED_UNIT_TERABYTE;
-			case 'c': return PED_UNIT_CYLINDER; 
+			case 'c': return PED_UNIT_CYLINDER;
 			case '%': return PED_UNIT_PERCENT;
 		}
 	}
@@ -482,17 +482,17 @@ parse_unit_suffix (const char* suffix, PedUnit suggested_unit)
 }
 
 /**
- * If \p str contains a valid description of a location on \p dev, then 
+ * If \p str contains a valid description of a location on \p dev, then
  * \p *sector is modified to describe the location and a geometry is created
  * in \p *range describing a 2 units large area centered on \p *sector.  If the
  * \p range as described here would be partially outside the device \p dev, the
  * geometry returned is the intersection between the former and the whole
  * device geometry.  If no units are specified, then the default unit is
- * assumed. 
+ * assumed.
  *
  * \throws PED_EXCEPTION_ERROR if \p str contains invalid description of a
  * location
- * \throws PED_EXCEPTION_ERROR if location described by \p str 
+ * \throws PED_EXCEPTION_ERROR if location described by \p str
  * is outside of the device \p dev->path
  *
  * \return \c 1 if \p str is a valid location description, \c 0 otherwise.
@@ -549,11 +549,11 @@ ped_unit_parse_custom (const char* str, const PedDevice* dev, PedUnit unit,
 	}
 	*sector = clip (dev, *sector);
 
-	ped_free (copy);
+	free (copy);
 	return 1;
 
 error_free_copy:
-	ped_free (copy);
+	free (copy);
 error:
 	*sector = 0;
 	if (range)
