@@ -4,7 +4,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <wchar.h>
@@ -26,8 +25,6 @@
 
 #include "strlist.h"
 
-#include <config.h>
-
 #ifdef ENABLE_NLS
 #       include <wchar.h>
 #else
@@ -44,12 +41,12 @@ typedef void Table;
 Table* table_new(int ncols);
 void table_destroy (Table* t);
 
-/* 
+/*
  * you must not free neither 'row' nor 'list'
  *      -- this will be done by table_destroy()
  */
 void table_add_row (Table* t, wchar_t** row);
 void table_add_row_from_strlist (Table* t, StrList* list);
-        
+
 wchar_t* table_render(Table* t);
 
