@@ -1,11 +1,11 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000, 2001, 2002, 2006, 2007
+    Copyright (C) 1999, 2000, 2001, 2002, 2006, 2007, 2008, 2009
     Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -14,10 +14,11 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <config.h>
+
 #include <parted/parted.h>
 #include <parted/debug.h>
 
@@ -28,10 +29,10 @@
 #include <unistd.h>
 #include <setjmp.h>
 
-#include <config.h>
 #include "command.h"
 #include "strlist.h"
 #include "ui.h"
+#include "error.h"
 
 #define N_(String) String
 #if ENABLE_NLS
@@ -154,17 +155,17 @@ struct siginfo_t {
 #  define ILL_BADSTK (INTMAX - 8)
 #endif
 
-char* prog_name = "GNU Parted " VERSION "\n";
+const char* prog_name = "GNU Parted " VERSION "\n";
 
-static char* banner_msg = N_(
+static const char* banner_msg = N_(
 "Welcome to GNU Parted! Type 'help' to view a list of commands.\n");
 
-static char* usage_msg = N_(
+static const char* usage_msg = N_(
 "Usage: parted [OPTION]... [DEVICE [COMMAND [PARAMETERS]...]...]\n"
 "Apply COMMANDs with PARAMETERS to DEVICE.  If no COMMAND(s) are given, "
 "run in\ninteractive mode.\n");
 
-static char* bug_msg = N_(
+static const char* bug_msg = N_(
 "\n\nYou found a bug in GNU Parted! Here's what you have to do:\n\n"
 "Don't panic! The bug has most likely not affected any of your data.\n"
 "Help us to fix this bug by doing the following:\n\n"
@@ -218,7 +219,7 @@ screen_width ()
 
 /* HACK: don't specify termcap separately - it'll annoy the users. */
 #ifdef HAVE_LIBREADLINE
-        width = tgetnum ("co");
+        width = tgetnum ((char *) "co");
 #endif
 
         if (width <= 0)
@@ -297,19 +298,9 @@ _dump_history (void)
 
 #endif /* HAVE_LIBREADLINE */
 
-static void
-mask_signal()
-{
-        sigset_t    curr;
-        sigset_t    prev;
-  
-        sigfillset(&curr);
-        sigprocmask(SIG_SETMASK, &curr, &prev);
-}
-
 /* Resets the environment by jumping to the initial state
  * saved during ui intitialisation.
- * Pass 1 as the parameter if you want to quit parted, 
+ * Pass 1 as the parameter if you want to quit parted,
  * 0 if you just want to reset to the command prompt.
  */
 static void
@@ -318,10 +309,10 @@ reset_env (int quit)
         int    in_readline = readline_state.in_readline;
 
         readline_state.in_readline = 0;
-        
+
         if (in_readline) {
                 putchar ('\n');
-                if (quit)    
+                if (quit)
                         exit (0);
 
                 siglongjmp (readline_state.jmp_state, 1);
@@ -339,15 +330,6 @@ sa_sigint_handler (int signum, siginfo_t* info, void *ucontext)
         reset_env (0);
 }
 
-/* Signal handler for SIGINT using 'signal'. */
-static void
-s_sigint_handler (int signum)
-{
-        signal (SIGINT, &s_sigint_handler);
-        mask_signal ();
-        sa_sigint_handler (signum, NULL, NULL);
-}
-
 /* Signal handler for SIGSEGV using 'sigaction'. */
 static void
 sa_sigsegv_handler (int signum, siginfo_t* info, void* ucontext)
@@ -383,15 +365,6 @@ sa_sigsegv_handler (int signum, siginfo_t* info, void* ucontext)
         abort ();
 }
 
-/* Signal handler for SIGSEGV using 'signal'. */
-static void
-s_sigsegv_handler (int signum)
-{
-        signal (SIGSEGV, &s_sigsegv_handler);
-        mask_signal ();
-        sa_sigsegv_handler (signum, NULL, NULL);
-}
-
 /* Signal handler for SIGFPE using 'sigaction'. */
 static void
 sa_sigfpe_handler (int signum, siginfo_t* info, void* ucontext)
@@ -410,12 +383,12 @@ sa_sigfpe_handler (int signum, siginfo_t* info, void* ucontext)
                         fputs(_("\nError: FPE_INTDIV (Integer: "
                                 "divide by zero)"), stdout);
                         break;
-        
+
                 case FPE_INTOVF:
                         fputs(_("\nError: FPE_INTOVF (Integer: "
                                 "overflow)"), stdout);
                         break;
-        
+
                 case FPE_FLTDIV:
                         fputs(_("\nError: FPE_FLTDIV (Float: "
                                 "divide by zero)"), stdout);
@@ -456,15 +429,6 @@ sa_sigfpe_handler (int signum, siginfo_t* info, void* ucontext)
         abort ();
 }
 
-/* Signal handler for SIGFPE using 'signal'. */
-static void
-s_sigfpe_handler (int signum)
-{
-        signal (SIGFPE, &s_sigfpe_handler);
-        mask_signal ();
-        sa_sigfpe_handler (signum, NULL, NULL);
-}
-
 /* Signal handler for SIGILL using 'sigaction'. */
 static void
 sa_sigill_handler (int signum, siginfo_t* info, void* ucontext)
@@ -524,10 +488,31 @@ sa_sigill_handler (int signum, siginfo_t* info, void* ucontext)
                                 "signal was encountered."), stdout);
                         break;
         }
-   
+
         abort ();
 }
 
+#ifndef SA_SIGINFO
+
+static void
+mask_signal()
+{
+        sigset_t    curr;
+        sigset_t    prev;
+
+        sigfillset(&curr);
+        sigprocmask(SIG_SETMASK, &curr, &prev);
+}
+
+/* Signal handler for SIGINT using 'signal'. */
+static void
+s_sigint_handler (int signum)
+{
+        signal (SIGINT, &s_sigint_handler);
+        mask_signal ();
+        sa_sigint_handler (signum, NULL, NULL);
+}
+
 /* Signal handler for SIGILL using 'signal'. */
 static void
 s_sigill_handler (int signum)
@@ -537,6 +522,25 @@ s_sigill_handler (int signum)
         sa_sigill_handler (signum, NULL, NULL);
 }
 
+/* Signal handler for SIGSEGV using 'signal'. */
+static void
+s_sigsegv_handler (int signum)
+{
+        signal (SIGSEGV, &s_sigsegv_handler);
+        mask_signal ();
+        sa_sigsegv_handler (signum, NULL, NULL);
+}
+
+/* Signal handler for SIGFPE using 'signal'. */
+static void
+s_sigfpe_handler (int signum)
+{
+        signal (SIGFPE, &s_sigfpe_handler);
+        mask_signal ();
+        sa_sigfpe_handler (signum, NULL, NULL);
+}
+#endif
+
 static char*
 _readline (const char* prompt, const StrList* possibilities)
 {
@@ -562,9 +566,17 @@ _readline (const char* prompt, const StrList* possibilities)
                 fputs (prompt, stdout);
                 fflush (stdout);
                 line = (char*) malloc (256);
-                if (fgets (line, 256, stdin) && strcmp (line, "") != 0)
-                        line [strlen (line) - 1] = 0;    /* kill trailing CR */
-                else {
+                if (fgets (line, 256, stdin) && strcmp (line, "") != 0) {
+#ifndef HAVE_LIBREADLINE
+                        /* Echo the input line, to be consistent with
+                           how readline-5.2 works.  */
+                        fputs (line, stdout);
+                        fflush (stdout);
+#endif
+                        /* kill trailing NL */
+                        if (strlen (line))
+                                line [strlen (line) - 1] = 0;
+                } else {
                         free (line);
                         line = NULL;
                 }
@@ -866,11 +878,15 @@ command_line_get_word (const char* prompt, const char* def,
                                 return result;
 
                         result_node = str_list_match (possibilities, result);
+                        if (result_node == NULL)
+                                error (0, 0, _("invalid token: %s"), result);
                         free (result);
                         if (result_node)
                                 return str_list_convert_node (result_node);
 
                         command_line_flush ();
+                        if (opt_script_mode)
+                                return NULL;
                 }
 
                 command_line_prompt_words (prompt, def, possibilities,
@@ -916,15 +932,15 @@ command_line_get_sector (const char* prompt, PedDevice* dev, PedSector* value,
         if (input && *value && !strcmp (input, def_str)) {
                 if (range) {
                         *range = ped_geometry_new (dev, *value, 1);
-                        ped_free (def_str);
+                        free (def_str);
                         return *range != NULL;
                 }
 
-                ped_free (def_str);
+                free (def_str);
                 return 1;
         }
 
-        ped_free (def_str);
+        free (def_str);
         if (!input) {
                 *value = 0;
                 if (range)
@@ -1003,16 +1019,16 @@ command_line_get_partition (const char* prompt, PedDisk* disk,
 {
         PedPartition*    part;
 
-        /* Flawed logic, doesn't seem to work?! 
+        /* Flawed logic, doesn't seem to work?!
         check = ped_disk_next_partition (disk, part);
         part  = ped_disk_next_partition (disk, check);
 
         if (part == NULL) {
 
-        *value = check;          
+        *value = check;
         printf (_("The (only) primary partition has "
                   "been automatically selected\n"));
-        return 1;          
+        return 1;
 
         } else {
         */
@@ -1110,7 +1126,7 @@ command_line_get_part_flag (const char* prompt, const PedPartition* part,
 
         if (flag_name) {
                 *flag = ped_partition_flag_get_by_name (flag_name);
-                ped_free (flag_name);
+                free (flag_name);
                 return 1;
         } else
                 return 0;
@@ -1213,7 +1229,7 @@ command_line_get_ex_opt (const char* prompt, PedExceptionOption options)
         PedExceptionOption    opt;
         char*                 opt_name;
 
-        for (opt = option_get_next (options, 0); opt; 
+        for (opt = option_get_next (options, 0); opt;
              opt = option_get_next (options, opt)) {
                 options_strlist = str_list_append_unique (options_strlist,
                                      _(ped_exception_get_option_string (opt)));
@@ -1331,6 +1347,7 @@ static int
 init_fs_type_str ()
 {
         PedFileSystemType*    walk;
+        PedFileSystemAlias*   alias_walk;
 
         fs_type_list = NULL;
 
@@ -1341,6 +1358,14 @@ init_fs_type_str ()
                 if (!fs_type_list)
                         return 0;
         }
+        for (alias_walk = ped_file_system_alias_get_next (NULL); alias_walk;
+             alias_walk = ped_file_system_alias_get_next (alias_walk))
+        {
+                fs_type_list = str_list_insert (fs_type_list,
+                                                alias_walk->alias);
+                if (!fs_type_list)
+                        return 0;
+        }
 
         return 1;
 }
@@ -1388,14 +1413,14 @@ init_ui ()
         sig_fpe.sa_sigaction = &sa_sigfpe_handler;
         sig_ill.sa_sigaction = &sa_sigill_handler;
 
-        sig_segv.sa_mask = 
-                sig_int.sa_mask = 
-                        sig_fpe.sa_mask = 
+        sig_segv.sa_mask =
+                sig_int.sa_mask =
+                        sig_fpe.sa_mask =
                                 sig_ill.sa_mask = curr;
-    
-        sig_segv.sa_flags = 
-                sig_int.sa_flags = 
-                        sig_fpe.sa_flags = 
+
+        sig_segv.sa_flags =
+                sig_int.sa_flags =
+                        sig_fpe.sa_flags =
                                 sig_ill.sa_flags = SA_SIGINFO;
 
         sigaction (SIGSEGV, &sig_segv, NULL);
@@ -1521,7 +1546,7 @@ non_interactive_mode (PedDevice** dev, Command* cmd_list[],
                         exit(1);
                         goto error;
                 }
-                
+
                 if (!command_run (cmd, dev))
                         goto error;
         }
